<?php

namespace HCM\FPA\Controllers;

use Zeedhi\Framework\DTO\Response\Notification;
use Zeedhi\Framework\DataSource\DataSet;
use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Doctrine\DBAL\Connection;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\HCMParameters;
use HCM\Util\FPA;
use HCM\Util\Repositories;
use Doctrine\ORM\EntityManager;
use HCM\FPA\ObrigacaoLegal\CNAB\PercorreCNAB;
use HCM\FPA\ObrigacaoLegal\CNAB\Banestes\Cnab021;
use HCM\FPA\ObrigacaoLegal\CNAB\Itau\Cnab341;
use HCM\FPA\ObrigacaoLegal\CNAB\Caixa\Cnab104;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab240;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab200;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab240237;
use HCM\FPA\ObrigacaoLegal\CNAB\Bradesco\Cnab200237;
use HCM\Geral\Controllers\RelatoriosHcmController;
use HCM\FPA\Services\CNABService;
use HCM\Util\DateUtil;

class CNABController extends Crud {
    
    private static $NM_PROCESSAMENTO = array('COMPETENCIA_MENSAL' => 'Competência Mensal');

    protected $CNABService;
    protected $dataSourceNameGet = 'getArquivoCNAB';
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO', 'P_DTMESCOMPETENC', 'P_CDOPERADOR');

    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager, Connection $connection, RelatoriosHcmController $relatoriosHcmController, CNABService $CNABService){
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        $this->entityManager = $entityManager;
        $this->connection = $connection;        
        $this->relatoriosHcmController = $relatoriosHcmController;
        $this->CNABService = $CNABService;
        parent::__construct($dataSourceManager);
    }

    public function getArquivoCNAB (DTO\Request\Filter $request, DTO\Response $response) {
        
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceNameGet);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData); 
        
        $rows = $dataSet->getRows();
        foreach($rows as $key => &$row){
            if($row['DSORDEM'] === 'LISTA_VINCULO'){
                $row['NRVINCULOM_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRVINCULOM_LIST'] as $chave => $value)
                    $row['NRVINCULOM_LIST'][$chave] = intval($value);
                
                $row['NMVINCULOM'] = explode(",", $row['NMVINCULOM']);
            }  
            
            if($row['DSORDEM'] === 'LISTA_LOTE'){
                $row['NRLOTEVINC_LIST'] = explode(",", $row['DSINICIO']);
                foreach($row['NRLOTEVINC_LIST'] as $chave => $value)
                    $row['NRLOTEVINC_LIST'][$chave] = intval($value);
                
                $row['LIST_NMLOTEVINC'] = explode(",", $row['LIST_NMLOTEVINC']);
            }  
            
            if($row['DSORDEM'] === 'LISTA_ESTRUTURA'){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }
            
            if($row['DSORDEM'] === 'LISTA_TOMADOR'){
                $row['LIST_NRESTRUTURAMSUP'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTURAMSUP'] as $chave => $value)
                    $row['LIST_NRESTRUTURAMSUP'][$chave] = intval($value);
                
                $row['NMESTRUTURAM'] = $row['NMESTRUTLEGAL'];
                $row['NMESTRUTURAMSUP'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']){
                    $row['LIST_NRESTRUTGEREN'] = explode(",", $row['DSFIM']);
                    foreach($row['LIST_NRESTRUTGEREN'] as $chave => $value)
                        $row['LIST_NRESTRUTGEREN'][$chave] = intval($value);
                    
                    $row['LIST_NMESTRUTGEREN'] = explode(",", $row['NMESTRUTGEREN']);
                }
            }  
            
            if($row['DSORDEM'] === "LISTA_NRESTRUTSIND"){
                $row['LIST_NRESTRUTLEGAL'] = explode(",", $row['DSINICIO']);
                foreach($row['LIST_NRESTRUTLEGAL'] as $chave => $value)
                    $row['LIST_NRESTRUTLEGAL'][$chave] = intval($value);
                
                $row['LIST_NMESTRUTLEGAL'] = explode(",", $row['NMESTRUTLEGAL']);
                
                if($row['DSFIM']) {
                    $row['LIST_NRESTRUTSIND'] = explode(',',$row['DSFIM']);
                    foreach($row['LIST_NRESTRUTSIND'] as $chave => $value)
                        $row['LIST_NRESTRUTSIND'][$chave] = intval($value);
                        
                    $row['LIST_NMESTRUTSIND'] = explode(",", $row['NMESTRUTSIND']);
                }
            }
            
        }
        $response->addDataSet(new DataSet($this->dataSourceNameGet, $rows));
    }
    
    public function setArquivoCNAB(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $this->dataSourceName = "fpa_arquivoCNAB";
            foreach ($request->getDataSet()->getRows() as &$row) {
                if (array_key_exists('NRFORMCNAB', $row)) {
                    if ($row['NRFORMCNAB'] === '') {
                        $row['NRFORMCNAB'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMCNAB', 1, 12, true); 
                    }
                } else {
                    $row['NRFORMCNAB'] = \HCM\Util\NovoCodigo::geraCodigo($row['NRORG'], 'FPA_FORMCNAB', 1, 12, true); 
                }
            
                if(!array_key_exists('DSINICIO', $row)){
                    $row['DSINICIO'] = null;
                }
    
                if(!array_key_exists('DSFIM', $row)){
                    $row['DSFIM'] = null;
                }
                if($row['DSORDEM'] === "LISTA_VINCULO"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                if($row['DSORDEM'] === "LISTA_LOTE"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                }
                if($row['DSORDEM'] === "LISTA_ESTRUTURA"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(isset($row['DSFIM']) && is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_TOMADOR"){
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(isset($row['DSFIM']) && is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                if($row['DSORDEM'] === "LISTA_NRESTRUTSIND") {
                    if(is_array($row['DSINICIO'])){
                        $row['DSINICIO'] = implode(',',$row['DSINICIO']);
                    }
                    if(is_array($row['DSFIM']) && count($row['DSFIM']) > 0){
                        $row['DSFIM'] = implode(',',$row['DSFIM']);
                    }else{
                        if(is_array($row['DSFIM'])){
                            $row['DSFIM'] = null;
                        }
                    }
                }
                
                $row['DSTIPOVINCULO'] = is_array($row['DSTIPOVINCULO'])?implode(',',$row['DSTIPOVINCULO']):null;
                $row['IDTPINSCHIERARQ'] = $row['IDTPINSCRESPONS'];
            }
            parent::save($request, $response);
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteArquivoCNAB(DTO\Request\DataSet $request, DTO\Response $response){
        try {
            $this->dataSourceName = "fpa_arquivoCNAB";
            parent::delete($request, $response);
            $response->addNotification(new Notification("row(s) deleted.", Notification::TYPE_SUCCESS));
        } catch (\Exception $e) {
            $response->setError(new DTO\Response\Error($e->getMessage(), $e->getCode()));
        }  
    }

    public function geraLoteCNAB(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $requestAux = clone $request;
            $row = $request->getDataSet()->getRows()[0];
            $rowAux = $requestAux->getDataSet()->getRows()[0];

            $dadosParaRetorno = array();
            $arquivos = array();
            $relatorios = array();

            if(!is_array($row['DSINICIO'])){
                $row['DSINICIO'] = explode(',', $row['DSINICIO']);
            }

            $responseAux1 = clone $response; // Cria um clone limpo do response

            $tomadores = $row['DSINICIO'];

            // Santander e Brasil necessitam de retrocesso a cada geração de analítico
            if($row['CDBANCO'] == '001' || $row['CDBANCO'] == '033') {
                foreach($tomadores as $key => $tomador) {
                    if($key != 0) {
                        $requestAux->getDataSet()->getRows()[0]['TP_PROCESSAMENTO'] = 'RETROCESSO';
                        $this->geraCNAB($requestAux, $responseAux2);
                        $requestAux->getDataSet()->getRows()[0]['TP_PROCESSAMENTO'] = 'COMPETENCIA_MENSAL';
                    }

                    // Somente coloca a data de geração do CNAB na segunda vez que o processo é realizado
                    $requestAux->getDataSet()->getRows()[0]['_preencheData'] = false;

                    $responseAux2 = clone $responseAux1; // Cria um clone do response limpo para utilizar na requisição
                    $requestAux->getDataSet()->getRows()[0]['DSINICIO'] = $tomador;
    
                    $this->geraCNAB($requestAux, $responseAux2);
                    
                    if(!empty($responseAux2->getDataSets())) {
                        foreach($responseAux2->getDataSets() as $dataRows) {
                            if($dataRows->getDataSourceName() == 'arquivo') {
                                $arquivos[] = $dataRows->getRows()['NMARQUIVO'];
                            } else {
                                foreach($dataRows->getRows() as &$dataSet) {
                                    $dataSet['DSDESCRICAO'] =  'Tomador: ' . $tomador . ' - ' . $dataSet['DSDESCRICAO'];
                                    $dadosParaRetorno[] = $dataSet;
                                }
                            } 
                        }
                    }
                    if(!empty($responseAux2->getMessages())) {
                        foreach($responseAux2->getMessages() as $message) {
                            // Somente coloca a informação do tomador caso não seja o link do relatório
                            if(substr($message->getMessage(), 0, 4) == 'http') {
                                $relatorios[] = $message->getMessage();
                            } else {
                                $mensagem = str_replace('<br>', '', $message->getMessage());
                                $dadosParaRetorno[] = array(
                                    "DSDESCRICAO" => 'Tomador: '. $tomador . ' - ' . $mensagem,
                                    "IDTIPOMSG"   => "M",
                                    "NRMSGCALC"   => null,
                                    "NRVINCULOM"  => null
                                );
                            }
                        }
                    }
                    if(!empty($responseAux2->getError())) {
                        $dadosParaRetorno[] = array(
                            "DSDESCRICAO" => 'Tomador: '. $tomador . ' - ' . $responseAux2->getError()->getMessage(),
                            "IDTIPOMSG"   => "E",
                            "NRMSGCALC"   => null,
                            "NRVINCULOM"  => null
                        );
                    }

                    //Depois da geração de todos os arquivos CNABs, é gerado todos os magnéticos novamente para os analíticos
                    $requestAux->getDataSet()->getRows()[0]['TP_PROCESSAMENTO'] = 'RETROCESSO';
                    $this->geraCNAB($requestAux, $responseAux2);
                    $requestAux->getDataSet()->getRows()[0]['TP_PROCESSAMENTO'] = 'COMPETENCIA_MENSAL';
                    
                    foreach($tomadores as $key => $tomador) {
                        $responseAux2 = clone $responseAux1;
                        $requestAux->getDataSet()->getRows()[0]['DSINICIO'] = $tomador;
                        $requestAux->getDataSet()->getRows()[0]['_preencheData'] = true;
        
                        $this->geraCNAB($requestAux, $responseAux2);
                    }
                }
            } else {
                foreach($tomadores as $tomador) {
                    $responseAux2 = clone $responseAux1; // Cria um clone do response limpo para utilizar na requisição
                    $requestAux->getDataSet()->getRows()[0]['DSINICIO'] = $tomador;
    
                    $this->geraCNAB($requestAux, $responseAux2);
    
                    if(!empty($responseAux2->getDataSets())) {
                        foreach($responseAux2->getDataSets() as $dataRows) {
                            if($dataRows->getDataSourceName() == 'arquivo') {
                                $arquivos[] = $dataRows->getRows()['NMARQUIVO'];
                            } else {
                                foreach($dataRows->getRows() as &$dataSet) {
                                    $dataSet['DSDESCRICAO'] =  'Tomador: ' . $tomador . ' - ' . $dataSet['DSDESCRICAO'];
                                    $dadosParaRetorno[] = $dataSet;
                                }
                            } 
                        }
                    }
                    if(!empty($responseAux2->getMessages())) {
                        foreach($responseAux2->getMessages() as $message) {
                            // Somente coloca a informação do tomador caso não seja o link do relatório
                            if(substr($message->getMessage(), 0, 4) == 'http') {
                                $relatorios[] = $message->getMessage();
                            } else {
                                $mensagem = str_replace('<br>', '', $message->getMessage());
                                $dadosParaRetorno[] = array(
                                    "DSDESCRICAO" => 'Tomador: '. $tomador . ' - ' . $mensagem,
                                    "IDTIPOMSG"   => "M",
                                    "NRMSGCALC"   => null,
                                    "NRVINCULOM"  => null
                                );
                            }
                        }
                    }
                    if(!empty($responseAux2->getError())) {
                        $dadosParaRetorno[] = array(
                            "DSDESCRICAO" => 'Tomador: '. $tomador . ' - ' . $responseAux2->getError()->getMessage(),
                            "IDTIPOMSG"   => "E",
                            "NRMSGCALC"   => null,
                            "NRVINCULOM"  => null
                        );
                    }
                }
            }
            

            // É necessário fazer o dataSet em separado pois a montagem do dataSet não funciona com mais de um array
            if(!empty($dadosParaRetorno)) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $dadosParaRetorno));
            }
            if(!empty($arquivos)) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $arquivos));
            }
            if(!empty($relatorios)) {
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('relatorio', $relatorios));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function geraCNAB(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            error_reporting(E_ERROR | E_PARSE);
            
            $dataset = $request->getDataSet()->getRows();
            $dataSet = $dataset[0];

            //$competencia = FPA::getDtCompetencia();
            //
            //$this->CNABService->deletaMagneticos($dataSet['NRORG'], $dataSet['NRTIPOMOVIMENT'], $dataSet['CDBANCO'], $competencia);
            
            switch($dataSet['CDBANCO']){ //Quando implemtado estes 2 bancos na classe Cnab240 este switch deve ser retirado mantendo o código abaixo
                case '001': //Banco do brasil 
                case '033': //Santander 
                    $percorreCNAB = new PercorreCNAB($this->entityManager, $this->hcmParameters, $dataSet);
                    $resultado = $percorreCNAB->percorre();
                    if($dataSet['TP_PROCESSAMENTO'] == 'RETROCESSO') {
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Retrocesso concluido com sucesso'));            
                    }
                    break;
                case '237': //Bradesco
                    
                    if($dataSet['IDLAYOUT'] == 200){
                        
                        $cnab = new Cnab200237($this->entityManager, $this->hcmParameters, $this->connection, $dataSet);
                        if($dataSet['TP_PROCESSAMENTO'] != 'RETROCESSO') {
                            $cnab->setCaminhoRelativo('CNAB_files');
                            $cnab->prepara();
                            $cnab->geraCabecalho();
                            $cnab->geraRegistros();
                            $resultado = $cnab->geraRodape();
                        } else {
                            $cnab->retrocede();
                            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Retrocesso concluido com sucesso'));
                        }
                        
                    }else{
                        $cnab = new Cnab240237($this->entityManager, $this->hcmParameters, $this->connection, $dataSet);
                        if($dataSet['TP_PROCESSAMENTO'] != 'RETROCESSO') {
                            $cnab->setCaminhoRelativo('CNAB_files');
                            $cnab->prepara();
                            $cnab->headerArquivo();
                            $cnab->headerLote();
                            $cnab->geraSegmentoA();
                            $cnab->geraTrailerLote();
                            $resultado = $cnab->geraTrailerArquivo();
                        } else {
                            $cnab->retrocede();
                            $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Retrocesso concluido com sucesso'));
                        }
                    }
                    break;
                case '021': //BANESTES
                    $cnab = new Cnab021($this->entityManager, $this->hcmParameters, $this->connection, $dataSet);
                    if($dataSet['TP_PROCESSAMENTO'] != 'RETROCESSO') {
                        $cnab->gerar();
                        $resultado = $cnab->gerarArquivo('cnab_banestes', 'txt');
                    } else {
                        $cnab->retroceder();
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Retrocesso concluido com sucesso'));            
                    }
                    break;
                case '341': //Itaú
                    $cnab = new Cnab341($this->entityManager, $this->hcmParameters, $this->connection, $dataSet);
                    if($dataSet['TP_PROCESSAMENTO'] != 'RETROCESSO') {
                        $cnab->gerar();
                        $resultado = $cnab->gerarArquivo('cnabitau', 'txt');
                    } else {
                        $cnab->retroceder();
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Retrocesso concluido com sucesso'));            
                    }
                    break;
                case '104': //Caixa
                    $cnab = new Cnab104($this->entityManager, $this->hcmParameters, $this->connection, $dataSet);
                    if($dataSet['TP_PROCESSAMENTO'] != 'RETROCESSO') {
                        $cnab->gerar();
                        $resultado = $cnab->gerarArquivo('CEF', 'rem');
                    } else  {
                        $cnab->retroceder();
                        $response->addMessage(new \Zeedhi\Framework\DTO\Response\Message('Retrocesso concluido com sucesso'));            
                    }
                    break;
                default:
                    throw new \Exception('Este banco ainda não está integrado ao sistema');
            }
            
            if ($dataSet['TP_PROCESSAMENTO'] != 'RETROCESSO' && $resultado['ABORT'] !== true){
                $filtro = array(
                    'P_NMARQUIVO'     => $resultado['ARQUIVO'],
                    'P_NRFORMCNAB'    => $dataSet['NRFORMCNAB'],
                    'P_MOVIMENTO'     => $dataSet['NRTIPOMOVIMENT'], 
                    'P_BANCO'         => $dataSet['CDBANCO'],
                    'P_ESTRUTURARESP' => $dataSet['NRESTRUTRESPONS'], 
                    'P_COMPETENCIA'   => $this->hcmParameters->getDtCompetencia(),
                    'P_NRESTRUTSUP'   => $dataSet['DSCRITERIO'] == 'LISTA_TOMADOR' ? $dataSet['DSINICIO'] : ''
                );
                $this->relatoriosHcmController->setRelatoriosHcmBackend($filtro, 'analitico_cnab', $request, $response);
            }
            
            if($resultado['ARQUIVO']){
                if($dataSet['SOMENTEANALITICO']) {
                    $message = new \Zeedhi\Framework\DTO\Response\Message('Analítico finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
                } else {
                    $row = array('NMARQUIVO' => FPA::getTempDirectory().'/'.$resultado['ARQUIVO']);
                    $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('arquivo', $row));
                    $message = new \Zeedhi\Framework\DTO\Response\Message('CNAB finalizado sem erros.', DTO\Response\Message::TYPE_MESSAGE);
                }
            }elseif ($resultado['ABORT'] === true) {
                $message = 'CNAB não gerado.<br>';
                foreach ($resultado['MENSAGENS'] as $mensagem) {
                    if ($mensagem['TIPO'] == \MsgCalculo::ERRO) {
                        $message .= strpos($message, $mensagem['MENSAGEM']) ? null : '<br> - ' . $mensagem['MENSAGEM'];
                    }
                }
                $message = new \Zeedhi\Framework\DTO\Response\Message($message, DTO\Response\Message::TYPE_ERROR);
            }else{
                $message = new \Zeedhi\Framework\DTO\Response\Message($dataSet['TP_PROCESSAMENTO'].' finalizado', DTO\Response\Message::TYPE_MESSAGE);
            }
            
            if($message instanceof \Zeedhi\Framework\DTO\Response\Message){
                $response->addMessage($message);
            }
            
            $messages = $this->entityManager->getRepository(Repositories::FPA_MSGCALCULO)->findBy(array('nrcalculofolha' => $resultado['CALCULO'], 'nrorg' => $this->hcmParameters->getNrorgTrabalho(), 'dsorigmsg' => 'CNAB'));
            if(!empty($messages)){
                $row = array();
                foreach ($messages as $mensagem) {
                    $row[] = array("NRVINCULOM" => $mensagem->getNrvinculom(), "NRMSGCALC" => $mensagem->getNrmsgcalc(), "DSDESCRICAO" => $mensagem->getDescricao(), "IDTIPOMSG" => $mensagem->getIdtipomsg());
                }
                $response->addDataSet(new \Zeedhi\Framework\DataSource\DataSet('msgCalculo', $row));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
}