<?php

namespace HCM\FPA\ObrigacaoLegal\CNAB;

use HCM\Util\Messages;
use HCM\Util\Repositories;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\TkDI;
use HCM\Util\MsgCalculo;
use HCM\Geral\Entity\GpepessoaEntity;
use HCM\Geral\Entity\GpevinculomEntity;
use HCM\Geral\Entity\GpemovimentacaoEntity;
use HCM\Geral\Entity\EnderecoparcEntity;
use HCM\Geral\Entity\BairroEntity;
use HCM\Geral\Entity\ContcorrparcEntity;
use HCM\FPA\Calculos\Geral\Estruturacao;
use HCM\Geral\Entity\EstruturamEntity;
use HCM\FPA\Entity\FpaitecalcfolhaEntity;
use HCM\FPA\Entity\FpacnabmagnetEntity;
use HCM\FPA\ObrigacaoLegal\CNAB\Santander\CNABSantander;
use HCM\FPA\ObrigacaoLegal\CNAB\Brasil\CNABBrasil;
use HCM\FPA\Registro\RegistroCNAB;
use HCM\FPA\Registro\FPARegistro;

class CNAB{

    private $_nrOrg;
    private $_nrOrgPadrao;
    private $_mesCompetencia;
    private $_dtInicio;
    private $_dtFim;
    private $_dtAux;
    private $_cnabMagnetico;
    private $_cnabMagneticoTrab;
    private $_valorTotalPagamento;
    private $_cnabSantander;
    private $_cnabBrasil;
    private $_registroCNAB;
    private $_itemCalculoFolha;
    private $_nrEventoPagamento;
    private $_nrItemCalcPagamento;
    private $_pessoa;
    private $_objPessoa;
    private $_vinculo;
    private $_nrTipoHierarqOrg;
    private $_movimentacao;
    private $_estrutura;
    private $_estruturacao;
    private $_nrEstruturaResp;
    private $_dtPagamento;
    private $_nrCalculoFolha;
    private $_cdBancoFormulario;
    private $_nrTpMovimentoFormulario;
    private $_enderecoParc;
    private $_municipio;
    private $_bairro;
    private $_contaCorrenteParceiro;
    private $_idUtilDtPgtoItem;
    private $_idIgnoraOcorrencia;
    private $_idDtGerUltCnab;
    private $_cdOperador;
    
    const TIPO_TRANSFERENCIA_LEGAL = '1';

    public function __construct(
        $nrOrg, $nrOrgPadrao, $mesCompetencia, $cnabMagnetico,
        $dataSet, $nrCalculoFolha, $entityManager, $hcmParameters
    ){
        
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->_nrOrg = $nrOrg;
        $this->_nrOrgPadrao = $nrOrgPadrao;
        $this->_mesCompetencia = $mesCompetencia;

        $this->_dtInicio = $this->_mesCompetencia;
        $this->_dtFim = DateUtil::getUltimoDiaDoMes($this->_mesCompetencia);

        $this->_itemCalculoFolha = new FpaitecalcfolhaEntity($entityManager, $hcmParameters);
        
        $this->_objPessoa = new GpepessoaEntity($entityManager, $hcmParameters);
        $this->_vinculoEntity = new GpevinculomEntity($entityManager, $hcmParameters);
        $this->_movimentacao = new GpemovimentacaoEntity($entityManager, $hcmParameters);

        $this->_enderecoParc = new EnderecoparcEntity($entityManager, $hcmParameters);
        $this->_bairro = new BairroEntity($entityManager, $hcmParameters);
        $this->_contaCorrenteParceiro = new ContcorrparcEntity($entityManager, $hcmParameters);
        $this->_cnabSantander = new CNABSantander($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $cnabMagnetico, $dataSet, $entityManager, $hcmParameters);
        $this->_cnabBrasil = new CNABBrasil($this->_nrOrg, $this->_nrOrgPadrao, $this->_mesCompetencia, $cnabMagnetico, $dataSet, $entityManager, $hcmParameters);
        $this->_registroCNAB = new RegistroCNAB();
        $this->_nrCalculoFolha = $nrCalculoFolha;
        
        $this->_estruturacao = new Estruturacao($entityManager, $hcmParameters);
        $this->_estrutura = new EstruturamEntity($entityManager, $hcmParameters);

        $this->_cnabMagnetico = $cnabMagnetico;
        $this->_cnabMagneticoTrab = new FpacnabmagnetEntity($entityManager, $hcmParameters);

        $this->_cdOperador = $hcmParameters->getCdOperador();
 
        $this->_inicializaVariaveis($dataSet);

    }

    private function _inicializaVariaveis($dataSet){
        $this->_estruturacao->carregaHierarquias($dataSet['NRHIERARQORG']);
        $this->_estruturacao->carregaTipoHierarquia($dataSet['NRHIERARQORG']);
        $this->_cdBancoFormulario = $dataSet['CDBANCO'];
        $this->_nrTpMovimentoFormulario = $dataSet['NRTIPOMOVIMENT'];
        $this->_nrTipoHierarqOrg = $dataSet['NRHIERARQORG'];
        $this->_nrEstruturaResp = $dataSet['NRESTRUTRESPONS'];
        $this->_dtPagamento = $dataSet['DTPAGAMENTO'];
        $this->_idUtilDtPgtoItem = $dataSet['IDUTILDTPGTOITEM'];
        $this->_idIgnoraOcorrencia = $dataSet['IDIGNORAOCORRENCIA'];
        $this->_idDtGerUltCnab = $dataSet['IDDTGERULTCNAB'];
        $this->_nrEventoPagamento = $dataSet['NREVENTOM'];
        $this->_valorTotalPagamento = 0;
        $this->_somenteAnalitico = isset($dataSet['SOMENTEANALITICO']) ? $dataSet['SOMENTEANALITICO'] : false;
        $this->_preencheData = isset($dataSet['_preencheData']) ? $dataSet['_preencheData'] : true;
    }

    public function preparaCNAB($vinculo){
        $this->_vinculo = $vinculo;
        $dtPagamento = $this->_idUtilDtPgtoItem == 'N' ? null : $this->_dtPagamento;
        if ($this->_idIgnoraOcorrencia == 'S') {
            foreach ($this->_nrCalculoFolha as $nrcalculofolha) {
                $dtGerUltCnab = $this->_vinculoEntity->getDtgerultcnab($vinculo, $nrcalculofolha, $this->_nrEventoPagamento, $dtPagamento);
                if (!is_null($dtGerUltCnab)) {
                    break;
                }
            }
        } else {
            $dtGerUltCnab = $this->_vinculoEntity->getDtgerultcnab($vinculo, $this->_nrCalculoFolha, $this->_nrEventoPagamento, $dtPagamento);
        }
        if (($this->_idDtGerUltCnab == 'N') or (($this->_idDtGerUltCnab == 'S') and is_null($dtGerUltCnab))) {
            $this->_pessoa = $this->_objPessoa->retornaPessoaAtual($this->_vinculo->getNrpessoa());
            if(!empty($this->_pessoa)){
                $this->_preparaEstrutura();
                $this->_preparaBancoPessoa();
                if ($this->_pessoa->banco === $this->_cdBancoFormulario) {
                    $this->_preparaPessoa();
                    $this->_preparaVinculo();
                    switch ($this->_estrutura->banco) {
                        case CNABSantander::CODIGO_BANCO_SANTANDER:
                            $this->_cnabSantander->geraCNAB($this->_estrutura, $this->_pessoa, $this->_vinculo);
                            break;
                        case CNABBrasil::CODIGO_BANCO_DO_BRASIL:
                            $this->_cnabBrasil->geraCNAB($this->_estrutura, $this->_pessoa, $this->_vinculo);
                            break;
                    }
                    $data = DateUtil::getDataAtual();
                    if($this->_itemCalculoFolha->getItem($this->_nrItemCalcPagamento) && !$this->_somenteAnalitico){
                        if($this->_preencheData) {
                            $this->_itemCalculoFolha->getItem($this->_nrItemCalcPagamento)->setDtgerultcnab($data);
                        }
                        $this->_itemCalculoFolha->persist();
                    }
                }
            }
        } else {
            MsgCalculo::getInstance()->adicionaMensagem('CNAB não gerado para vínculo ' . $this->_vinculo->getNmvinculom() . ' por já possuir CNAB gerado em '.$dtGerUltCnab.'.');
        }
    }

    private function _preparaEstrutura() {
        $movimentacao = $this->_estruturacao->getMovimentacaoVinculoPorTipo($this->_vinculo->getNrvinculom(), self::TIPO_TRANSFERENCIA_LEGAL);
        
        if (is_object($movimentacao) === true) {
            $this->_estrutura = $this->_estruturacao->getEstruturaSuperior($movimentacao);
			if (is_object($this->_estrutura) === true) {
                $estrutura = $this->_estruturacao->retornaEstrutura($this->_estrutura->getNrestruturam());
                $this->_estrutura->historico = $estrutura->historico;
			} else {
                MsgCalculo::getInstance()->adicionaMensagem('Deve ser escolhido um tipo de hierarquia cuja estrutura relacionada seja uma estrutura legal.', MsgCalculo::ERRO, true);
            }

        } else {
            $this->_estrutura = null;
            MsgCalculo::getInstance()->adicionaMensagem('Movimentação legal não encontrada para o vínculo ' . $this->_vinculo->getNrvinculom(), MsgCalculo::ERRO);
        }
        
        if ($this->_estrutura !== null) {
            $contaCorrenteEstrutura = $this->entityManager->getRepository(Repositories::CONTCORRPARC)->findOneBy(array('nrparcnegocio' => $this->_estrutura->historico->getNrparcnegocio(), 'nrorg' => $this->_nrOrg, 'cdbanco' => $this->_cdBancoFormulario));

            if (is_object($contaCorrenteEstrutura) == true) {
                $this->_estrutura->banco = $contaCorrenteEstrutura->getCdbanco();
                $this->_estrutura->agencia = $contaCorrenteEstrutura->getCdagencia();
                $this->_estrutura->contaCorrente = $contaCorrenteEstrutura->getCdcontcorr();
            } else {
                $this->_estrutura->banco = null;
                $this->_estrutura->agencia = null;
                $this->_estrutura->contaCorrente = null;
                MsgCalculo::getInstance()->adicionaMensagem('Parceiro de negocio '.$this->_estrutura->historico->getNrparcnegocio().' da estrutura nao possui conta corrente.', MsgCalculo::ERRO, true);
            }
            
			if ($this->_estrutura->historico) {				
				$cnpjEstrutura = $this->_estrutura->historico->getCdcnpjestrut();
				$cpfEstrutura = $this->_estrutura->historico->getCdcpfestrutura();
			} else {
                MsgCalculo::getInstance()->adicionaMensagem('Vínculo '.$this->_vinculo->getNrvinculom(). ' com problema na movimentação legal.', MsgCalculo::ERRO, true);
			}
			
            if (empty($cnpjEstrutura) === false) {
                $this->_estrutura->nrTpInscricao = RegistroCNAB::CNPJ;
                $this->_estrutura->nrInscricao = $cnpjEstrutura;
            } elseif (empty($cpfEstrutura) === false) {
                $this->_estrutura->nrTpInscricao = RegistroCNAB::CPF;
                $this->_estrutura->nrInscricao = $cpfEstrutura;
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Estrutura sem número de inscrição.', MsgCalculo::ERRO, true);
            }
            if (empty($cnpjEstrutura) === false) {			
				$endereco = $this->_estruturacao->determinaEnderecoEstrutura($this->_estrutura);
				$this->_estrutura->endereco = $endereco;
			}
        }	
    }

    private function _preparaBancoPessoa() {
      $contaCorrentePessoa = $this->_contaCorrenteParceiro->retornaContaCorrentePorParceiroBanco($this->_pessoa->getNrparcnegocio(), $this->_cdBancoFormulario);
        if (is_object($contaCorrentePessoa) == true) {
            $this->_pessoa->banco = $contaCorrentePessoa->getCdbanco();
            $this->_pessoa->agencia = $contaCorrentePessoa->getCdagencia();
            $this->_pessoa->contaCorrente = $contaCorrentePessoa->getCdcontcorr();
        } else {
            $this->_pessoa->banco = null;
            $this->_pessoa->agencia = null;
            $this->_pessoa->contaCorrente = null;
        }
    }

    private function _preparaPessoa(){

        $nrParceiroNegocio = $this->_pessoa->getNrparcnegocio();
        $this->_pessoa->nrTpInscricao = RegistroCNAB::CPF;
        $enderecoParc = $this->_enderecoParc->retornaEnderecoParceiroPorTipo(
            $this->_nrOrg, $nrParceiroNegocio, Estruturacao::ENDERECO_PRINCIPAL
        );

        if (is_object($enderecoParc) === true) {
            $enderecoParc->separaDsEndereco();

            $municipio = $this->entityManager->getRepository(Repositories::MUNICIPIO)->findOneBy(array('cdpais' => $enderecoParc->getCdpais(),
            'sgestado' => $enderecoParc->getSgestado(),'cdmunicipio'=> $enderecoParc->getCdmunicipio()));
            
            if (is_object($municipio) === true) {
                $this->_pessoa->municipio = $municipio->getNmmunicipio();
            } else {
                MsgCalculo::getInstance()->adicionaMensagem('Verificar municipio do endereço de '.$this->_pessoa->historico->getNmpessoa());
                $this->_pessoa->municipio = null;
            }
            $this->_pessoa->endereco    = $enderecoParc->getDsendereco();
            $this->_pessoa->numero      = $enderecoParc->getNrimovelendereco();
            $this->_pessoa->complemento = ltrim($enderecoParc->getComplemento());
            $this->_pessoa->bairro      = $this->_bairro->defineBairro($enderecoParc);
            $this->_pessoa->estado      = $enderecoParc->getSgestado();
            $this->_pessoa->cep         = $enderecoParc->getNrcependereco();
        } else {
            $this->_pessoa->endereco = null;
            $this->_pessoa->numero = null;
            $this->_pessoa->complemento = null;
            $this->_pessoa->bairro = null;
            $this->_pessoa->municipio = null;
            $this->_pessoa->estado = null;
            $this->_pessoa->cep = null;
            MsgCalculo::getInstance()->adicionaMensagem('Endereço não parametrizado para '.$this->_pessoa->historico->getNmpessoa());
        }
    }

    private function _preparaVinculo(){
        $this->_definePagamentoCnab();
    }

    public function modificaSequencia($linha, $sequencia){

        $linha = $this->_registroCNAB->alteraDado(
            $linha, (string)$sequencia, (9 -1), 5, RegistroCNAB::CARACTER_ZERO, RegistroCNAB::PREENCHE_ESQUERDA
        );
        return $linha;
    }

    public function modificaLinhasConsolidacao($tipoRegistro, $linha){

        switch($this->_cdBancoFormulario) {
            case CNABSantander::CODIGO_BANCO_SANTANDER:
                $linha = $this->_cnabSantander->modificaLinhasConsolidacaoSantander($tipoRegistro, $linha, $this->_valorTotalPagamento);
                break;
            case CNABBrasil::CODIGO_BANCO_DO_BRASIL:
                $linha = $this->_cnabBrasil->modificaLinhasConsolidacaoBrasil($tipoRegistro, $linha, $this->_valorTotalPagamento);
                break;
        }
        return $linha;
    }

    private function _modificaSequenciaArquivo() {
        $this->_cnabMagnetico->modificaSequenciaArquivoRemessa($this->_mesCompetencia, $this->_nrOrg, $this->_cdBancoFormulario, $this->_cdOperador);
    }
    
    public function retroceder(){
        $this->_cnabMagnetico->retroceder($this->_nrOrg, $this->_mesCompetencia, $this->_cdBancoFormulario, $this->_nrTpMovimentoFormulario, $this->_cdOperador);
    }

    private function _definePagamentoCnab(){
        $dtPagamento = $this->_idUtilDtPgtoItem == 'N' ? null : $this->_dtPagamento;
        
        if ($this->_idIgnoraOcorrencia == 'S') {
            $valorPagamento = 0;
            $dataPagamento = false;
            
            foreach ($this->_nrCalculoFolha as $nrcalculofolha) {
                $this->_itemCalculoFolha->retornaItensPorVinculoAndEvento($this->_nrOrg, $this->_vinculo->getNrvinculom(), $nrcalculofolha, $this->_nrEventoPagamento);
                
                $this->_nrItemCalcPagamento = $this->_itemCalculoFolha->procuraEventoPorNreventoDtocorrencia($this->_nrEventoPagamento, $dtPagamento);
                
                if ($this->_nrItemCalcPagamento !== -1) {
                    $valorPagamento += (float)$this->_itemCalculoFolha->retornaValor($this->_nrItemCalcPagamento);
                    $dataPagamento = true;
                }
            }
            
            $this->_vinculo->valorPagamento = $valorPagamento;
            $this->_vinculo->dataPagamento = $dataPagamento ? $this->_dtPagamento : FPARegistro::CARACTER_ZERO;
            $this->_valorTotalPagamento += $valorPagamento;
        } else {
            $this->_itemCalculoFolha->retornaItensPorVinculoAndEvento($this->_nrOrg, $this->_vinculo->getNrvinculom(), $this->_nrCalculoFolha, $this->_nrEventoPagamento);
            
            $this->_nrItemCalcPagamento = $this->_itemCalculoFolha->procuraEventoPorNreventoDtocorrencia($this->_nrEventoPagamento, $dtPagamento);
    
            if ($this->_nrItemCalcPagamento !== -1) {
                $this->_vinculo->valorPagamento = (float)$this->_itemCalculoFolha->retornaValor($this->_nrItemCalcPagamento);
                $this->_vinculo->dataPagamento = $this->_dtPagamento;
                $this->_valorTotalPagamento += $this->_vinculo->valorPagamento;
            } else {
                $this->_vinculo->valorPagamento = FPARegistro::CARACTER_ZERO;
                $this->_vinculo->dataPagamento = FPARegistro::CARACTER_ZERO;
            }
        }
    }

    public function getValorTotalPagamento(){
        return $this->_valorTotalPagamento;
    }

    public function setValorTotalPagamento($valor){
        $this->_valorTotalPagamento = $valor;
    }
}