<?php

namespace HCM\Geral\Controllers;

use Zeedhi\Framework\Controller\Crud;
use Zeedhi\Framework\DataSource\Manager;
use Zeedhi\Framework\DataSource\ParameterBag;
use Zeedhi\Framework\DataSource\FilterCriteria;
use Doctrine\ORM\EntityManager;
use Zeedhi\Framework\DTO;
use Zeedhi\Framework\DTO\Request;
use HCM\Util\Repositories;
use HCM\Util\HCMParameters;
use Teknisa\Model\Geral\Entities\Base\Histpadr;
use HCM\Util\Util;

class HistoricoPadraoController extends Crud {

    protected $dataSourceName = 'histpadr';
    
    protected $parameterBagColumns = array('P_NRORG', 'P_NRORG_PADRAO');
    public function __construct (Manager $dataSourceManager, ParameterBag $parameterBag, HCMParameters $hcmParameters, EntityManager $entityManager){
        $this->entityManager = $entityManager;
        $this->parameterBag = $parameterBag;
        $this->hcmParameters = $hcmParameters;
        parent::__construct($dataSourceManager);
    }

    public function getHistoricoPadrao(DTO\Request\Filter $request, DTO\Response $response) {
        $filterData = $request->getFilterCriteria();
        $filterData->setDataSourceName($this->dataSourceName);
        $newFilterData = $this->populateParameterBag($filterData);
        $dataSet = $this->dataSourceManager->findBy($newFilterData);
        $response->addDataSet($dataSet);
    }
    
    public function setHistoricoPadrao(DTO\Request\DataSet $request, DTO\Response $response) {
        try {
            $row = $request->getDataSet()->getRows()[0];
            $cdhistpadrao = $row['CDHISTPADR'];
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            
            //caso a row seja nova, é feito uma busca por Hisóricos Padrões já existentes para verificar se já existe um com o código informado
            $obj = null;
            if ($row['__is_new'] == true)
                $obj = $this->entityManager->getRepository(Repositories::HISTORICOPADRAO)->findOneBy(array("cdhistpadr" => $cdhistpadrao, "nrorg" => $nrorg));
                
            if (!$obj){
                foreach ($request->getDataSet()->getRows() as &$row) {
                    
                    if(!array_key_exists('DTULTATUHIPD', $row)){
                        $row['DTULTATUHIPD'] = $row['DTULTATU'];
                    }
                    if(isset($row['DSHISTPADR'])){
                        $row['DSHISTPADR'] = Util::removeCaracteresEspeciais($row['DSHISTPADR']);
                    } 
                }
                parent::save($request, $response);
            } else {
                $response->addMessage(new DTO\Response\Message("Já existe um histórico com o código informado!", DTO\Response\Message::TYPE_ERROR));
            }
        } catch(\Exception $e) {
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }
    }
    
    public function deleteHistoricoPadrao(DTO\Request\DataSet $request, DTO\Response $response){
        try{
            parent::delete($request, $response);
        } catch(\Exception $e){
            $response->setError(new \Zeedhi\Framework\DTO\Response\Error($e->getMessage(), $e->getCode()));
        }    
    }

    protected function processConditionValue($condition) {
        return $condition['value'];
    }
    protected function populateParameterBag(FilterCriteria $filterCriteria) {

        $conditions = array();
        foreach ($filterCriteria->getConditions() as $condition) {
            $columnName = $condition['columnName'];
            if (in_array($columnName, $this->parameterBagColumns)) {
                $this->parameterBag->set($columnName, $this->processConditionValue($condition));
            } else {
                $conditions[] = $condition;
            }
        }
        $newFilterCriteria = new FilterCriteria(
            $filterCriteria->getDataSourceName(),
            $conditions,
            $filterCriteria->getPage(),
            $filterCriteria->getPageSize()
        );
        return $newFilterCriteria;
    }
} 