<?php

namespace HCM\Geral\Logic\Apolo;

use Doctrine\ORM\EntityManager;
use HCM\Geral\Logic\ESocialLogic;
use HCM\Util\DateUtil;
use HCM\Util\FPA;
use HCM\Util\HCMParameters;
use HCM\Util\Repositories;
use Zeedhi\Framework\Session\Session;
use Zeedhi\Queue\Connection;

class ConsumerTransmissaoEsocial extends \Zeedhi\Queue\Consumer{

    protected static $queueName = "/queue/esocial_transmissao";
    
    private $entityManager;
    private $hcmParameters;
    private $DIR_PHP;

    public function __construct(Connection $connection, EntityManager $entityManager, $hcmParameters, $DIR_PHP){
        parent::__construct($connection);
        $this->entityManager = $entityManager;
        $this->hcmParameters = $hcmParameters;
        $this->DIR_PHP = $DIR_PHP;
    }
    
    /**
     * @param string $message
     * @return void
     */
    public function consume($message) {
        $DIR_PHP = $this->DIR_PHP;
        $params = json_decode($message, true);
        
        $nrorg = $params['nrorg'];
        $cdoperador = $params['cdoperador'];
        $dtmescompetenc = $params['dtmescompetenc'];
        $nrestruturam = $params['nrestruturam'];
        $nrtpambiente = $params['nrtpambiente'];
        $idgrupo = $params['idgrupo'];
        $nresoconsumer = $params['nresoconsumer'];
        $nrlote = $params['nrlote'];
        $lote = json_encode($params['lote']);
            
        $start = microtime(true);
        
        $this->log("\nStart lote {$nrorg}.{$nrlote} at " . date("H:i:s",$start));
        
        $console = shell_exec("cd {$this->dir} && $DIR_PHP esocialTransmiteLote.php {$nrorg} {$cdoperador} {$dtmescompetenc} {$nrestruturam} {$nrtpambiente} {$idgrupo} {$nresoconsumer} {$nrlote} {$lote}");
        $this->log($console);
        
        $end = microtime(true);
        $time = $end-$start;
        
        /*
        $duration = $endtime-$starttime;
        $hours = (int)($duration/60/60);
        $minutes = (int)($duration/60)-$hours*60;
        $seconds = (int)$duration-$hours*60*60-$minutes*60;
        */
        
        $this->log("Processed lote {$nrorg}.{$nrlote} at " . date("H:i:s",$end) . " in " . $time);
        $this->log('------------------------');
    }

    /**
     * @param string $message
     * @return void
     */
    public function exec($nrorg, $cdoperador, $dtmescompetenc, $nrestruturam, $nrtpambiente, $idgrupo, $nresoconsumer, $nrlote, $lote) {
        try {
            ini_set('memory_limit', '2048M');
            set_time_limit(0);
            
            $pid = getmypid();
        
            $this->setParameters($nrorg, $cdoperador, $dtmescompetenc);
            
            $lote = json_decode($lote, true);
            
            //$this->entityManager->getRepository(Repositories::ESO_CONSUMER)->addPidprocess($nrorg, $nresoconsumer, $pid);
            $this->log("START pid ".$pid." qtdLote ".count($lote));
            
            $logic = new ESocialLogic($this->entityManager, $this->hcmParameters, $nrestruturam, $nrtpambiente);
            $msgRetorno = $logic->preparaEnviarLoteEvento($idgrupo, $lote, $nrlote);
            $msgValidacao = $logic->getMessages();
            
            if (count($msgValidacao) > 0) {
                $this->log($msgValidacao);
            } elseif ($msgRetorno) {
                $this->log($msgRetorno);
            } else {
                $this->log("Envio de evento não disponível");
            }
            
            $this->entityManager->getRepository(Repositories::ESO_CONSUMER)->addNrtotprocess($nrorg, $nresoconsumer, count($lote));
            $this->log("END pid ".$pid." qtdLote ".count($lote));
        } catch(\Exception $e) {
            $this->log($e->getMessage());
        }
        
        $this->clearLoteEnvios($nrorg, $nrlote, $idgrupo);
    }
    
    protected function clearLoteEnvios($nrorg, $nrlote, $idgrupo) {
        if ($idgrupo == ESocialLogic::IDGRUPO_CADASTRO) {
            $this->log("clearLoteEnviosCadastros");
            $this->entityManager->getRepository(Repositories::ESO_CONSUMER)->clearLoteEsocadastro($nrorg, $nrlote);
        }
        if ($idgrupo == ESocialLogic::IDGRUPO_NAOPERIODICO) {
            $this->log("clearLoteEnviosNaoPeriodicos");
            $this->entityManager->getRepository(Repositories::ESO_CONSUMER)->clearLoteEsonaoperiodico($nrorg, $nrlote);
        }
        if ($idgrupo == ESocialLogic::IDGRUPO_PERIODICO) {
            $this->log("clearLoteEnviosPeriodicos");
            $this->entityManager->getRepository(Repositories::ESO_CONSUMER)->clearLoteEsoperiodico($nrorg, $nrlote);
        }
    }

    protected function setParameters($nrorg, $cdoperador, $dtmescompetenc) {
        
        $session = new Session();
        $session->set('NRORG', $nrorg);
        
        $parametersXml = simplexml_load_file(__DIR__.'/../../../../../config/parameters.xml');
        $projectVersion = (string)$parametersXml->parameters->parameter[2];
        
        $fileServerXml = simplexml_load_file(__DIR__.'/../../../../../config/fileServer.xml');
        $httpInterface = (string)$fileServerXml->parameters->parameter[1];
        $filesTempPath = (string)$fileServerXml->parameters->parameter[2];
        
        $this->hcmParameters = new HCMParameters($session, FPA::getTempDirectory(), $httpInterface, $projectVersion, $filesTempPath);
        
        FPA::setSession($this->hcmParameters->getSession());
        
        $paramOrg = $this->entityManager->getRepository(Repositories::GPE_PARAMORG)->findOneBy(array('nrorg' => $nrorg));
        $operador = $this->entityManager->getRepository(Repositories::OPERADOR)->findOneBy(array('cdoperador' => $cdoperador));
        $paramOper = $this->entityManager->getRepository(Repositories::GPE_PARAMOPER)->findOneBy(array('cdoperador' => $cdoperador));
        
        if ($paramOrg) {
            $nrtpestrutlegal = $paramOrg->getNrtpestrutlegal();
            $nrtpestrutgeren = $paramOrg->getNrtpestrutgeren();
        } else {
            $nrtpestrutlegal = 1;
            $nrtpestrutgeren = 6;
        }
        
        $paramsHCM = array(
            "cdoperador"        => $cdoperador,
            "CDOPERADOR"        => $cdoperador,
            "nmoperador"        => $operador->getNmoperador(),
            "nrestruttrab"      => null,
            "nrorgpadrao"       => 0,
            "nrorg"             => $nrorg,
            "nrorgtrab"         => $nrorg,
            "dtmescompetenc"    => $dtmescompetenc,
            "dtmescompetencdiacorrente" => $dtmescompetenc,
            "nrtipoestrutlegal" => $nrtpestrutlegal,
            "nrtipoestrutgeren" => $nrtpestrutgeren,
            "idambientesal"     => 'REAL',
            "nrocorrecal"       => $paramOper->getNrocorrecal(),
            "nrtpmodalidcal"    => $paramOper->getNrtpmodalidcal(),
            "nrtpmodalidsal"    => $paramOper->getNrtpmodalidsal(),
            "dataatual"         => DateUtil::getDataAtual(),
            "dtultimodiames"    => DateUtil::getUltimoDiaDoMes(DateUtil::getDataAtual())->format('d/m/Y'),
            "dtultimodiacompetencia" => DateUtil::getUltimoDiaDoMes(DateUtil::getDataAtual())->format('d/m/Y'),
        );
        
        foreach ($paramsHCM as $key => $value) {
            $this->hcmParameters->set($key, $value);
        }
    }

    protected function log($message) {
        echo $message, "\n";
    }
}