<?php
namespace HCM\Geral\Repository;

use Doctrine\DBAL\Connection;
use Doctrine\ORM\EntityManager;

class PreFechamentoFgtsRepository {
    
    public function __construct(Connection $connection, EntityManager $entityManager) {
        $this->connection = $connection;
        $this->entityManager = $entityManager;
        $this->conferenciaFgts = new ConferenciaFgts($entityManager);
        $this->informacoesSinteticasFgts = new InformacoesSinteticasFgts($entityManager);
        $this->informacoesSobreBasesCalculoFgts = new InformacoesSobreBasesCalculoFgts($entityManager);
    }
        
    public function getConferenciaFgts($parameters) {
        return $this->conferenciaFgts->getConferenciaFgts($parameters);
    }
    
    public function getInformacoesSinteticasFgts($parameters) {
        return $this->informacoesSinteticasFgts->getInformacoesSinteticasFgts($parameters);
    }
    
    public function getInformacoesSobreBasesCalculoFgts($parameters) {
        return $this->informacoesSobreBasesCalculoFgts->getInformacoesSobreBasesCalculoFgts($parameters);
    }
}

class ConferenciaFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getConferenciaFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$conferenciaFgts);
        $sql->execute(
            array(
                'P_NRORG'          => $parameters['NRORG'],
                'P_MODALIDADECAL'  => 1,
                'P_IGNORAOCORR'    => 'N',
                'P_NRESTRUTGEREN'  => $parameters['NRESTRUTGEREN'],
                'P_OCORRECAL'      => 1,
                'P_COMPETENCIA'    => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL'  => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'      => '',
                'P_NRTABELA'       => $parameters['NRTABELA']
            )
        );
        return $sql->fetchAll();
    }
}

class InformacoesSinteticasFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getInformacoesSinteticasFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$informacoesSinteticasFgts);
        $sql->execute(
            array(
                'P_NRORG'         => $parameters['NRORG'],
                'P_MODALIDADECAL' => 1,
                'P_IGNORAOCORR'   => 'N',
                'P_NRESTRUTGEREN' => $parameters['NRESTRUTGEREN'],
                'P_OCORRECAL'     => 1,
                'P_COMPETENCIA'   => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL' => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'     => '',
                'P_NRTABELA'      => $parameters['NRTABELA']
            )
        );
        return $sql->fetchAll();
    }
}

class InformacoesSobreBasesCalculoFgts {
    
    public function __construct(EntityManager $entityManager) {
        $this->entityManager = $entityManager;
    }
    
    public function getInformacoesSobreBasesCalculoFgts($parameters) {
        $sql = $this->entityManager->getConnection()->prepare(Queries::$informacoesSobreBasesCalculoFgts);
        $sql->execute(
            array(
                'P_NRORG'         => $parameters['NRORG'],
                'P_COMPETENCIA'   => $parameters['DTMESCOMPETENC'],
                'P_NRESTRUTLEGAL' => is_array($parameters['NRESTRUTLEGAL']) ? implode(",",$parameters['NRESTRUTLEGAL']) : $parameters['NRESTRUTLEGAL'],
                'P_NRVINCULO'     => '',
                'P_NRTABELA'      => $parameters['NRTABELA'],
                'P_MODALIDADECAL' => 1,
                'P_IGNORAOCORR'   => 'N',
                'P_OCORRECAL'     => 1
            )
        );
        return $sql->fetchAll();
    }
}

class Queries {
    
    static $conferenciaFgts = "
      SELECT * FROM ( 
        SELECT RES.VINCULO,
              RES.DTRESCISAOVINC,
              RES.MATRICULA,
              RES.REGISTRO,
              PE.NRCPFPESSOA,
              RES.NOME,
              RES.NRLEGAL,
              RES.NRGEREN,
              RES.RAZAOLEGAL,
              RES.CNPJLEGAL,
              RES.INTLEGAL,
              RES.RAZAOGEREN,
              RES.CNPJGEREN,
              RES.INTGEREN,
              ENVIO.IDSTATUS,
              ENVIO.NRTABELA,
              :P_COMPETENCIA PERIODO,
              SUBSTR(RES.CNPJLEGAL, 0,8) CNPJRAIZ,
              RES.CDESOCIAL CDCATEGESOC,
              (RES.CDESOCIAL || ' - ' || RES.DSCATEGESOC) DSCATEGESOC,
              RES.BASE BASE_FGTS,
              RES.FGTS DEPOSITO_FGTS,
              SUM(NVL(ENVIO.VRBASEFGTSMENSAL + NVL(ENVIO.VRBASEFGTS13,0), 0) + NVL(ESONP.VRBASEFGTSMENSAL + NVL(ESONP.VRBASEFGTS13,0), 0) + NVL(ESOPE.VRBASEFGTSMENSAL + NVL(ESOPE.VRBASEFGTS13,0), 0)) BASE_FGTS_ESOCIAL,
              SUM(NVL(ENVIO.VRDEPOSITOFGTS + NVL(ENVIO.VRDEPOSITOFGTS13,0), 0) + NVL(ESONP.VRDEPOSITOFGTS + NVL(ESONP.VRDEPOSITOFGTS13,0), 0) + NVL(ESOPE.VRDEPOSITOFGTS + NVL(ESOPE.VRDEPOSITOFGTS13,0), 0)) DEPOSITO_FGTS_ESOCIAL,
              RES.BASE - SUM(NVL(ENVIO.VRBASEFGTSMENSAL + NVL(ENVIO.VRBASEFGTS13,0), 0) + NVL(ESONP.VRBASEFGTSMENSAL + NVL(ESONP.VRBASEFGTS13,0), 0) + NVL(ESOPE.VRBASEFGTSMENSAL + NVL(ESOPE.VRBASEFGTS13,0), 0)) DIFERENCA_BASE,
              RES.FGTS - SUM(NVL(ENVIO.VRDEPOSITOFGTS + NVL(ENVIO.VRDEPOSITOFGTS13,0), 0) + NVL(ESONP.VRDEPOSITOFGTS + NVL(ESONP.VRDEPOSITOFGTS13,0), 0) + NVL(ESOPE.VRDEPOSITOFGTS + NVL(ESOPE.VRDEPOSITOFGTS13,0), 0)) DIFERENCA_DEPOSITO
          FROM (SELECT VM.NRORG, VM.NRPESSOA,
                      VM.NRVINCULOM      VINCULO,
                      VM.DTRESCISAOVINC,
                      VM.CDMATRICULA     MATRICULA,
                      VM.DSLIVROADMISSAO REGISTRO,
                      VM.NMVINCULOM      NOME,
                      VM.NRTIPOVINCULOM,
                      SUM(CASE WHEN ICF.NREVENTOM IN (3060, 3180, 3210,  3120) THEN ICF.VREVENTOFOLHA ELSE 0 END) BASE,
                      SUM(CASE WHEN ICF.NREVENTOM IN (2010, 2050, 2080, 2020) THEN ICF.VREVENTOFOLHA ELSE 0 END) FGTS,
                      CF.DTOCORRENCIA,
                      VH.NRESTRUTLEGAL   NRLEGAL,
                      VH.NRESTRUTGEREN   NRGEREN,
                      EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                      EHL.CDCNPJESTRUT   CNPJLEGAL,
                      EML.CDINTESTRUTURA INTLEGAL,
                      EHG.NMRAZSOCESTRUT RAZAOGEREN,
                      EHG.CDCNPJESTRUT   CNPJGEREN,
                      EMG.CDINTESTRUTURA INTGEREN,
                      CATEGTRAB.DSCATEGESOC, 
                      CATEGTRAB.CDESOCIAL, 
                      EP.NRTPAMBIENTE
                  FROM GPE_VINCULOM VM
                  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                      AND VH.NRVINCULOM = VM.NRVINCULOM
                                      AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                  FROM GPE_VINCULOH
                                                                WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                  AND NRORG      = VM.NRORG
                                                                  AND NRVINCULOM = VM.NRVINCULOM )
                  /*ESTRUTURA LEGAL*/
                  JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                    AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                  LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = VH.NRORG
                                          AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                          AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                      FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG = VH.NRORG
                                                                        AND NRESTRUTURAM = EML.NRESTRUTURAM )
                  /*ESTRUTURA GERENCIAL*/
                  JOIN ESTRUTURAM EMG ON EMG.NRORG = VH.NRORG
                                    AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN
                  LEFT JOIN ESTRUTURAH EHG ON EHG.NRORG = VH.NRORG
                                          AND EHG.NRESTRUTURAM = EMG.NRESTRUTURAM
                                          AND EHG.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                      FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG        = VH.NRORG
                                                                        AND NRESTRUTURAM = EMG.NRESTRUTURAM )
                  /*CÁLCULO DE FOLHA*/
                  JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = VM.NRORG
                                          AND ICF.NRVINCULOM  = VM.NRVINCULOM
                                          AND ICF.NREVENTOM IN (3060, 2010, 3180, 2050, 3210, 2080, 3120, 2020)
                  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                          AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                          AND ((CF.NRTIPOMOVIMENT IN (1,2,10,19) AND :P_NRTABELA = 1) OR
                                             (CF.NRTIPOMOVIMENT = 4 AND :P_NRTABELA = 2) OR
                                             (CF.NRTIPOMOVIMENT IN (1,2,4,10,19) AND :P_NRTABELA IS NULL))
                                          AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                          AND CF.DTOCORRENCIA = :P_COMPETENCIA
                                          AND (CF.NROCORRECAL = :P_OCORRECAL OR :P_IGNORAOCORR = 'S')
                  /*CATEGORIA*/
                  JOIN GPE_VINCULOEMPREG VINCEMPREG ON VINCEMPREG.NRVINCULOEMPREG = VM.NRVINCULOEMPREG 
                  JOIN GPE_CATEGTRABESOC CATEGTRAB ON CATEGTRAB.NRCATEGESOC = VINCEMPREG.NRCATEGESOC 
                  JOIN ESO_PARAMETRO EP ON EP.NRORG = EML.NRORG 
                    AND EP.NRESTRUTURAM = EML.NRESTRUTURAM 
                  /**/
                WHERE VM.NRORG = :P_NRORG
                  AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                  AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                  AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL OR 1 = 1)
                  AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                  AND ( INSTR(','||:P_NRESTRUTGEREN||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :P_NRESTRUTGEREN IS NULL )
                GROUP BY VM.NRORG, VM.NRPESSOA, VM.NRVINCULOM,
                  VM.DTRESCISAOVINC,
                  VM.CDMATRICULA,
                  VM.DSLIVROADMISSAO,
                  VM.NMVINCULOM,
                  VM.NRTIPOVINCULOM,
                  /*ICF.NREVENTOM,*/
                  CF.DTOCORRENCIA,
                  VH.NRESTRUTLEGAL,
                  VH.NRESTRUTGEREN,
                  EHL.NMRAZSOCESTRUT,
                  EHL.CDCNPJESTRUT,
                  EML.CDINTESTRUTURA,
                  EHG.NMRAZSOCESTRUT,
                  EHG.CDCNPJESTRUT,
                  EMG.CDINTESTRUTURA,
                  CATEGTRAB.DSCATEGESOC,
                  CATEGTRAB.CDESOCIAL,
                  EP.NRTPAMBIENTE
                ) RES
        
        LEFT JOIN ESO_ENVIOCADASTROS ENVIO ON ENVIO.NRORG = RES.NRORG
                                          AND TRUNC(ENVIO.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                                          AND ENVIO.NRTPAMBIENTE = RES.NRTPAMBIENTE
                                          AND ENVIO.IDATIVO = 'S' AND ENVIO.NRENVIOEXCL IS NULL 
                                          AND ENVIO.IDSTATUS IN('C','A','M')
                                          AND (ENVIO.NRTABELA = :P_NRTABELA OR :P_NRTABELA IS NULL)
                                          AND EXISTS (
                                            SELECT 1
                                            FROM ESO_EVENTO EVESO
                                            WHERE EVESO.NREVENTO = ENVIO.NREVENTO
                                              AND EVESO.NMEVENTO IN ('S-2299','S-2399','S-1200')
                                              AND ((EVESO.NMEVENTO = 'S-2299' AND RES.NRTIPOVINCULOM = 1 AND RES.DTRESCISAOVINC >= :P_COMPETENCIA AND ENVIO.NRCHAVE = RES.VINCULO) OR
                                                   (EVESO.NMEVENTO = 'S-2399' AND RES.NRTIPOVINCULOM IN (2,3,10) AND RES.DTRESCISAOVINC >= :P_COMPETENCIA AND ENVIO.NRCHAVE = RES.VINCULO) OR
                                                   (EVESO.NMEVENTO = 'S-1200' AND (RES.DTRESCISAOVINC IS NULL OR RES.DTRESCISAOVINC < :P_COMPETENCIA) AND ENVIO.NRCHAVE = RES.NRPESSOA))
                                        )
        LEFT JOIN (SELECT ESONP.DTMESCOMPETENC,
                    ESONP.NRTPAMBIENTE,
                    ESONP.NRVINCULOM,
                    SUM(NVL(NPFGTS.VRBASEFGTSMENSAL,0)) AS VRBASEFGTSMENSAL,
                    SUM(NVL(NPFGTS.VRBASEFGTS13,0)) AS VRBASEFGTS13,
                    SUM(NVL(NPFGTS.VRDEPOSITOFGTS, 0)) AS VRDEPOSITOFGTS,
                    SUM(NVL(NPFGTS.VRDEPOSITOFGTS13,0)) AS VRDEPOSITOFGTS13,
                    ESONP.NRENVIO,
                    EHPN.CDCNPJESTRUT
                FROM ESO_NAOPERIODICO ESONP 
                JOIN ESTRUTURAH EHPN ON EHPN.NRORG   = ESONP.NRORG
                                          AND EHPN.NRESTRUTURAM = ESONP.NRESTRUTURAM
                                          AND EHPN.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = ESONP.NRORG
                                                                      AND NRESTRUTURAM = EHPN.NRESTRUTURAM )
                JOIN ESO_NAOPERIODFGTS NPFGTS ON NPFGTS.NRORG = ESONP.NRORG
                                        AND NPFGTS.NRENVIO = ESONP.NRENVIO
                WHERE ESONP.NRORG = :P_NRORG
                AND ESONP.IDATIVO = 'S' AND ESONP.NRENVIOEXCL IS NULL
                AND ESONP.IDSTATUS IN('C','A')
                AND (ESONP.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESONP.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                AND ESONP.NMEVENTO IN ('S-2299','S-2399')
                GROUP BY ESONP.DTMESCOMPETENC, ESONP.NRTPAMBIENTE, ESONP.NRVINCULOM,ESONP.NRENVIO, EHPN.CDCNPJESTRUT
            )ESONP ON TRUNC(ESONP.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                   AND ESONP.NRTPAMBIENTE = RES.NRTPAMBIENTE
                   AND ESONP.NRVINCULOM = RES.VINCULO
                   AND SUBSTR(ESONP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8)
                   
        LEFT JOIN (SELECT ESOPE.DTMESCOMPETENC,
                ESOPE.NRTPAMBIENTE,
                ESOPE.NRCHAVE,
                SUM(NVL(PEFGTS.VRBASEFGTSMENSAL,0)) AS VRBASEFGTSMENSAL,
                SUM(NVL(PEFGTS.VRBASEFGTS13,0)) AS VRBASEFGTS13,
                SUM(NVL(PEFGTS.VRDEPOSITOFGTS, 0)) AS VRDEPOSITOFGTS,
                SUM(NVL(PEFGTS.VRDEPOSITOFGTS13,0)) AS VRDEPOSITOFGTS13,
                ESOPE.NRENVIO,
                EHPE.CDCNPJESTRUT 
                FROM ESO_PERIODICO ESOPE   
                JOIN ESTRUTURAH EHPE ON EHPE.NRORG   = ESOPE.NRORG
                                          AND EHPE.NRESTRUTURAM = ESOPE.NRESTRUTURAM
                                          AND EHPE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = ESOPE.NRORG
                                                                      AND NRESTRUTURAM = EHPE.NRESTRUTURAM )
                JOIN ESO_PERIODFGTS PEFGTS ON PEFGTS.NRORG = ESOPE.NRORG
                                        AND PEFGTS.NRENVIO = ESOPE.NRENVIO
                WHERE ESOPE.NRORG = :P_NRORG
                AND ESOPE.IDATIVO = 'S' AND ESOPE.NRENVIOEXCL IS NULL
                AND ESOPE.IDSTATUS IN('C','A')
                AND (ESOPE.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESOPE.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                AND ESOPE.NMEVENTO = 'S-1200'
                GROUP BY ESOPE.DTMESCOMPETENC,ESOPE.NRTPAMBIENTE,ESOPE.NRCHAVE,ESOPE.NRENVIO, EHPE.CDCNPJESTRUT 
            )ESOPE ON TRUNC(ESOPE.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                   AND ESOPE.NRTPAMBIENTE = RES.NRTPAMBIENTE
                   AND ESOPE.NRCHAVE = RES.NRPESSOA
                   AND SUBSTR(ESOPE.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8)
                
        LEFT JOIN ESTRUTURAH EMP ON EMP.NRORG = ENVIO.NRORG
                                AND EMP.NRESTRUTURAM = ENVIO.NRESTRUTURAM
                                AND EMP.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                            FROM ESTRUTURAH
                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                              AND NRORG        = EMP.NRORG
                                                              AND NRESTRUTURAM = EMP.NRESTRUTURAM )
                                AND SUBSTR(EMP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8) AND EMP.CDCNPJESTRUT IS NOT NULL
        JOIN GPE_PESSOAH PE ON PE.NRPESSOA = RES.NRPESSOA 
                          AND PE.NRORG = RES.NRORG 
                          AND PE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                      FROM GPE_PESSOAH 
                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                        AND NRORG = PE.NRORG
                                                        AND NRPESSOA = PE.NRPESSOA )
        LEFT JOIN ESO_EVENTO EVESO ON EVESO.NREVENTO = ENVIO.NREVENTO
      
      GROUP BY RES.VINCULO,
                RES.DTRESCISAOVINC,
                RES.MATRICULA,
                RES.REGISTRO,
                PE.NRCPFPESSOA,
                RES.NOME,
                RES.NRLEGAL,
                RES.NRGEREN,
                RES.RAZAOLEGAL,
                RES.CNPJLEGAL,
                RES.INTLEGAL,
                RES.RAZAOGEREN,
                RES.CNPJGEREN,
                RES.INTGEREN,
                ENVIO.IDSTATUS,
                ENVIO.NRTABELA,
                RES.CDESOCIAL,
                (RES.CDESOCIAL || ' - ' || RES.DSCATEGESOC),
                RES.BASE,
                RES.FGTS,
                ENVIO.VRBASEFGTSMENSAL,
                ENVIO.VRBASEFGTS13,
                ENVIO.VRDEPOSITOFGTS,
                ENVIO.VRDEPOSITOFGTS13
      )
      ORDER BY CNPJRAIZ, CNPJLEGAL, NOME
    ";
    
    static $informacoesSinteticasFgts = "
      SELECT 
      * 
      FROM (
        SELECT 
          NRLEGAL,
          RAZAOLEGAL,
          ('('||NRLEGAL||') '||RAZAOLEGAL) ESTABELECIMENTO,
          CNPJLEGAL,
          INTLEGAL,
          CNPJRAIZ,
          SUM(DEPOSITO_FGTS) DEPOSITO_FGTS,
          SUM(DEPOSITO_FGTS_ESOCIAL) DEPOSITO_FGTS_ESOCIAL,
          SUM(DEPOSITO_FGTS_13) DEPOSITO_FGTS_13,
          SUM(DEPOSITO_FGTS_ESOCIAL_13) DEPOSITO_FGTS_ESOCIAL_13,
          /*SUM(FGTS_RESCISAO) FGTS_RESCISAO,
          SUM( (CASE WHEN DTRESCISAOVINC IS NOT NULL THEN DEPOSITO_FGTS_ESOCIAL ELSE 0 END) ) RESCISAO_ESOCIAL,*/
          SUM(DIFERENCA_DEPOSITO) DIFERENCA_DEPOSITO
          FROM ( 
              SELECT RES.VINCULO,
                    RES.DTRESCISAOVINC,
                    RES.MATRICULA,
                    RES.REGISTRO,
                    PE.NRCPFPESSOA,
                    RES.NOME,
                    RES.NRLEGAL,
                    RES.NRGEREN,
                    RES.RAZAOLEGAL,
                    RES.CNPJLEGAL,
                    RES.INTLEGAL,
                    RES.RAZAOGEREN,
                    RES.CNPJGEREN,
                    RES.INTGEREN,
                    ENVIO.IDSTATUS,
                    ENVIO.NRTABELA,
                    RES.CDESOCIAL CDCATEGESOC,
                    (RES.CDESOCIAL || ' - ' || RES.DSCATEGESOC) DSCATEGESOC,
                    :P_COMPETENCIA PERIODO,
                    SUBSTR(RES.CNPJLEGAL, 0,8) CNPJRAIZ,
                    SUM(RES.FGTS_MES) DEPOSITO_FGTS,
                    SUM(NVL(ENVIO.VRDEPOSITOFGTS,0)  + NVL(ENVIO.VRDEPOSITOFGTS13,0) + NVL(ESONP.VRDEPOSITOFGTS,0) + NVL(ESONP.VRDEPOSITOFGTS13,0) + NVL(ESOPE.VRDEPOSITOFGTS,0) + NVL(ESOPE.VRDEPOSITOFGTS13,0) ) DEPOSITO_FGTS_ESOCIAL,
                    SUM(RES.FGTS_MES + NVL(RES.FGTS_13,0)) - SUM(NVL(ENVIO.VRDEPOSITOFGTS  + NVL(ENVIO.VRDEPOSITOFGTS13,0),0) + NVL(ESONP.VRDEPOSITOFGTS,0) + NVL(ESONP.VRDEPOSITOFGTS13,0) + NVL(ESOPE.VRDEPOSITOFGTS,0) + NVL(ESOPE.VRDEPOSITOFGTS13,0)) DIFERENCA_DEPOSITO,
                    SUM(RES.FGTS_13) DEPOSITO_FGTS_13,
                    SUM(ENVIO.VRDEPOSITOFGTS13) DEPOSITO_FGTS_ESOCIAL_13
                    /*SUM(RES.FGTS_RESCISAO) FGTS_RESCISAO*/
                FROM (SELECT VM.NRORG, VM.NRPESSOA,
                            VM.NRVINCULOM      VINCULO,
                            VM.DTRESCISAOVINC,
                            VM.CDMATRICULA     MATRICULA,
                            VM.DSLIVROADMISSAO REGISTRO,
                            VM.NMVINCULOM      NOME,
                            VM.NRTIPOVINCULOM,
                            CF.DTOCORRENCIA,
                            VH.NRESTRUTLEGAL   NRLEGAL,
                            VH.NRESTRUTGEREN   NRGEREN,
                            EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                            EHL.CDCNPJESTRUT   CNPJLEGAL,
                            EML.CDINTESTRUTURA INTLEGAL,
                            EHG.NMRAZSOCESTRUT RAZAOGEREN,
                            EHG.CDCNPJESTRUT   CNPJGEREN,
                            EP.NRTPAMBIENTE,
                            CATEGTRAB.DSCATEGESOC, 
                            CATEGTRAB.CDESOCIAL, 
                            EMG.CDINTESTRUTURA INTGEREN,
                            SUM(CASE WHEN ICF.NREVENTOM IN (2010) THEN ICF.VREVENTOFOLHA ELSE 0 END) FGTS_MES,
                            SUM(CASE WHEN ICF.NREVENTOM IN (2020, 2080) THEN ICF.VREVENTOFOLHA ELSE 0 END) FGTS_13
                        FROM GPE_VINCULOM VM
                        JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                            AND VH.NRVINCULOM = VM.NRVINCULOM
                                            AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                        FROM GPE_VINCULOH
                                                                      WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG      = VM.NRORG
                                                                        AND NRVINCULOM = VM.NRVINCULOM )
                        /*ESTRUTURA LEGAL*/
                        JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                          AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                        LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG = VH.NRORG
                                                AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                                AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                            FROM ESTRUTURAH
                                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                              AND NRORG = VH.NRORG
                                                                              AND NRESTRUTURAM = EML.NRESTRUTURAM )
                        /*ESTRUTURA GERENCIAL*/
                        JOIN ESTRUTURAM EMG ON EMG.NRORG = VH.NRORG
                                          AND EMG.NRESTRUTURAM = VH.NRESTRUTGEREN
                        LEFT JOIN ESTRUTURAH EHG ON EHG.NRORG = VH.NRORG
                                                AND EHG.NRESTRUTURAM = EMG.NRESTRUTURAM
                                                AND EHG.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                            FROM ESTRUTURAH
                                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                              AND NRORG        = VH.NRORG
                                                                              AND NRESTRUTURAM = EMG.NRESTRUTURAM )
                      /*CÁLCULO DE FOLHA*/
                      JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = VM.NRORG
                                              AND ICF.NRVINCULOM = VM.NRVINCULOM
                                              AND ICF.NREVENTOM IN (2010, 2050, 2080, 2020)
                      JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                              AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                              AND ((CF.NRTIPOMOVIMENT IN (1,2,10,19) AND :P_NRTABELA = 1) OR
                                                 (CF.NRTIPOMOVIMENT = 4 AND :P_NRTABELA = 2) OR
                                                 (CF.NRTIPOMOVIMENT IN (1,2,4,10,19) AND :P_NRTABELA IS NULL))
                                              AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                              AND CF.DTOCORRENCIA = :P_COMPETENCIA
                                              AND (CF.NROCORRECAL = :P_OCORRECAL OR :P_IGNORAOCORR = 'S')
                      /*CATEGORIA*/
                      JOIN GPE_VINCULOEMPREG VINCEMPREG ON VINCEMPREG.NRVINCULOEMPREG = VM.NRVINCULOEMPREG 
                      JOIN GPE_CATEGTRABESOC CATEGTRAB ON CATEGTRAB.NRCATEGESOC = VINCEMPREG.NRCATEGESOC 
                      JOIN ESO_PARAMETRO EP ON EP.NRORG = EML.NRORG 
                        AND EP.NRESTRUTURAM = EML.NRESTRUTURAM 
                        
                      WHERE VM.NRORG = :P_NRORG
                        AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                        AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                        AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL OR 1 = 1)
                        AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                        AND ( INSTR(','||:P_NRESTRUTGEREN||',', ','||VH.NRESTRUTGEREN||',' ) > 0 OR :P_NRESTRUTGEREN IS NULL )
                      GROUP BY VM.NRORG, VM.NRPESSOA, VM.NRVINCULOM,
                        VM.DTRESCISAOVINC,
                        VM.CDMATRICULA,
                        VM.DSLIVROADMISSAO,
                        VM.NMVINCULOM,
                        VM.NRTIPOVINCULOM,
                        /*ICF.NREVENTOM,*/
                        CF.DTOCORRENCIA,
                        VH.NRESTRUTLEGAL,
                        VH.NRESTRUTGEREN,
                        EHL.NMRAZSOCESTRUT,
                        EHL.CDCNPJESTRUT,
                        EML.CDINTESTRUTURA,
                        EHG.NMRAZSOCESTRUT,
                        EHG.CDCNPJESTRUT,
                        CATEGTRAB.DSCATEGESOC,
                        CATEGTRAB.CDESOCIAL,
                        EMG.CDINTESTRUTURA,
                        EP.NRTPAMBIENTE
                      ) RES
              
              LEFT JOIN ESO_ENVIOCADASTROS ENVIO ON ENVIO.NRORG = RES.NRORG
                                                AND TRUNC(ENVIO.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                                                AND ENVIO.NRTPAMBIENTE = RES.NRTPAMBIENTE
                                                AND ENVIO.IDATIVO = 'S' AND ENVIO.NRENVIOEXCL IS NULL 
                                                AND ENVIO.IDSTATUS IN('C','A','M')
                                                AND (ENVIO.NRTABELA = :P_NRTABELA OR :P_NRTABELA IS NULL)
                                                AND EXISTS (
                                                    SELECT 1
                                                    FROM ESO_EVENTO EVESO
                                                    WHERE EVESO.NREVENTO = ENVIO.NREVENTO
                                                      AND EVESO.NMEVENTO IN ('S-2299','S-2399','S-1200')
                                                      AND ((EVESO.NMEVENTO = 'S-2299' AND RES.NRTIPOVINCULOM = 1 AND RES.DTRESCISAOVINC >= :P_COMPETENCIA AND ENVIO.NRCHAVE = RES.VINCULO) OR
                                                           (EVESO.NMEVENTO = 'S-2399' AND RES.NRTIPOVINCULOM IN (2,3,10) AND RES.DTRESCISAOVINC >= :P_COMPETENCIA AND ENVIO.NRCHAVE = RES.VINCULO) OR
                                                           (EVESO.NMEVENTO = 'S-1200' AND (RES.DTRESCISAOVINC IS NULL OR RES.DTRESCISAOVINC < :P_COMPETENCIA) AND ENVIO.NRCHAVE = RES.NRPESSOA))
                                                )
              LEFT JOIN (SELECT ESONP.DTMESCOMPETENC,
                    ESONP.NRTPAMBIENTE,
                    ESONP.NRVINCULOM,
                    SUM(NVL(NPFGTS.VRDEPOSITOFGTS, 0)) AS VRDEPOSITOFGTS,
                    SUM(NVL(NPFGTS.VRDEPOSITOFGTS13,0)) AS VRDEPOSITOFGTS13,
                    ESONP.NRENVIO,
                    EHPN.CDCNPJESTRUT
                  FROM ESO_NAOPERIODICO ESONP 
                  JOIN ESTRUTURAH EHPN ON EHPN.NRORG   = ESONP.NRORG
                                            AND EHPN.NRESTRUTURAM = ESONP.NRESTRUTURAM
                                            AND EHPN.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                        WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                        AND NRORG        = ESONP.NRORG
                                                                        AND NRESTRUTURAM = EHPN.NRESTRUTURAM )
                  JOIN ESO_NAOPERIODFGTS NPFGTS ON NPFGTS.NRORG = ESONP.NRORG
                                          AND NPFGTS.NRENVIO = ESONP.NRENVIO
                  WHERE ESONP.NRORG = :P_NRORG
                  AND ESONP.IDATIVO = 'S' AND ESONP.NRENVIOEXCL IS NULL
                  AND ESONP.IDSTATUS IN('C','A')
                  AND (ESONP.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESONP.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                  AND ESONP.NMEVENTO IN ('S-2299','S-2399')
                  GROUP BY ESONP.DTMESCOMPETENC, ESONP.NRTPAMBIENTE, ESONP.NRVINCULOM,ESONP.NRENVIO, EHPN.CDCNPJESTRUT
                )ESONP ON TRUNC(ESONP.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                     AND ESONP.NRTPAMBIENTE = RES.NRTPAMBIENTE
                     AND ESONP.NRVINCULOM = RES.VINCULO
                     AND SUBSTR(ESONP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8)
                   
              LEFT JOIN (SELECT ESOPE.DTMESCOMPETENC,
                    ESOPE.NRTPAMBIENTE,
                    ESOPE.NRCHAVE,
                    SUM(NVL(PEFGTS.VRDEPOSITOFGTS, 0)) AS VRDEPOSITOFGTS,
                    SUM(NVL(PEFGTS.VRDEPOSITOFGTS13,0)) AS VRDEPOSITOFGTS13,
                    ESOPE.NRENVIO,
                    EHPE.CDCNPJESTRUT 
                    FROM ESO_PERIODICO ESOPE   
                    JOIN ESTRUTURAH EHPE ON EHPE.NRORG   = ESOPE.NRORG
                                              AND EHPE.NRESTRUTURAM = ESOPE.NRESTRUTURAM
                                              AND EHPE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                          WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                          AND NRORG        = ESOPE.NRORG
                                                                          AND NRESTRUTURAM = EHPE.NRESTRUTURAM )
                    JOIN ESO_PERIODFGTS PEFGTS ON PEFGTS.NRORG = ESOPE.NRORG
                                            AND PEFGTS.NRENVIO = ESOPE.NRENVIO
                    WHERE ESOPE.NRORG = :P_NRORG
                    AND ESOPE.IDATIVO = 'S' AND ESOPE.NRENVIOEXCL IS NULL
                    AND ESOPE.IDSTATUS IN('C','A')
                    AND (ESOPE.NRTABELA = :P_NRTABELA OR (:P_NRTABELA = 1 AND ESOPE.NRTABELA IS NULL) OR :P_NRTABELA IS NULL)
                    AND ESOPE.NMEVENTO = 'S-1200'
                    GROUP BY ESOPE.DTMESCOMPETENC,ESOPE.NRTPAMBIENTE,ESOPE.NRCHAVE,ESOPE.NRENVIO, EHPE.CDCNPJESTRUT 
                  )ESOPE ON TRUNC(ESOPE.DTMESCOMPETENC, 'MM') = RES.DTOCORRENCIA
                       AND ESOPE.NRTPAMBIENTE = RES.NRTPAMBIENTE
                       AND ESOPE.NRCHAVE = RES.NRPESSOA
                       AND SUBSTR(ESOPE.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8)
              LEFT JOIN ESTRUTURAH EMP ON EMP.NRORG = ENVIO.NRORG
                                      AND EMP.NRESTRUTURAM = ENVIO.NRESTRUTURAM
                                      AND EMP.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                                  FROM ESTRUTURAH
                                                                  WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                    AND NRORG        = EMP.NRORG
                                                                    AND NRESTRUTURAM = EMP.NRESTRUTURAM )
                                      AND SUBSTR(EMP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8) AND EMP.CDCNPJESTRUT IS NOT NULL
              JOIN GPE_PESSOAH PE ON PE.NRPESSOA = RES.NRPESSOA 
                                AND PE.NRORG = RES.NRORG 
                                AND PE.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) 
                                                            FROM GPE_PESSOAH 
                                                            WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                              AND NRORG = PE.NRORG
                                                              AND NRPESSOA = PE.NRPESSOA )
              LEFT JOIN ESO_EVENTO EVESO ON EVESO.NREVENTO = ENVIO.NREVENTO
                                        AND EVESO.NMEVENTO IN ('S-2299','S-2399','S-1200')
            /*WHERE ((EVESO.NMEVENTO = 'S-1200' AND ENVIO.NRCHAVE = RES.NRPESSOA) OR (EVESO.NMEVENTO IN ('S-2299','S-2399') AND ENVIO.NRCHAVE = RES.VINCULO))
              AND SUBSTR(EMP.CDCNPJESTRUT,1,8) = SUBSTR(RES.CNPJLEGAL,1,8) AND EMP.CDCNPJESTRUT IS NOT NULL*/
            GROUP BY RES.VINCULO,
                      RES.DTRESCISAOVINC,
                      RES.MATRICULA,
                      RES.REGISTRO,
                      PE.NRCPFPESSOA,
                      RES.NOME,
                      RES.NRLEGAL,
                      RES.NRGEREN,
                      RES.RAZAOLEGAL,
                      RES.CNPJLEGAL,
                      RES.INTLEGAL,
                      RES.RAZAOGEREN,
                      RES.CNPJGEREN,
                      RES.INTGEREN,
                      ENVIO.IDSTATUS,
                      ENVIO.NRTABELA,
                      RES.FGTS_MES,
                      RES.CDESOCIAL,
                      (RES.CDESOCIAL || ' - ' || RES.DSCATEGESOC),
                      /*RES.FGTS_RESCISAO,*/
                      RES.FGTS_13,
                      ENVIO.VRDEPOSITOFGTS,
                      ENVIO.VRDEPOSITOFGTS13
            )
    GROUP BY NRLEGAL, RAZAOLEGAL, CNPJLEGAL, INTLEGAL, CNPJRAIZ)
    ORDER BY CNPJRAIZ, CNPJLEGAL
    ";
    
    static $informacoesSobreBasesCalculoFgts = "
      SELECT * FROM ( 
        SELECT 
            NOME, 
            NRCPFPESSOA,
            NRLEGAL,
            RAZAOLEGAL,
            ('('||NRLEGAL||') '||RAZAOLEGAL||' '||CNPJLEGAL) ESTABELECIMENTO,
            CNPJLEGAL,
            PRI.CDTERCEIRO,
            CNPJRAIZ,
            PRI.FGTS BASE,
            (CASE WHEN PRI.DTRESCISAOVINC IS NULL THEN PRI.VRBASEFGTSMENSAL ELSE 0 END) BASE_ESOCIAL,
            PRI.FGTS_13 BASE_13,
            (CASE WHEN PRI.DTRESCISAOVINC IS NULL THEN PRI.VRBASEFGTS13 ELSE 0 END) BASE_13_ESOCIAL,
            PRI.FGTS_RESCISAO,
            (CASE WHEN PRI.DTRESCISAOVINC IS NOT NULL THEN PRI.VRBASEFGTSMENSAL + NVL(PRI.VRBASEFGTS13,0) ELSE 0 END) RESCISAO_ESOCIAL,
            PRI.NRESTRUTLOTACAO,
            ( '('||PRI.NRESTRUTLOTACAO||') '||'FPAS '||PRI.CDFPAS||' '||'Terceiros '||PRI.CDTERCEIRO ) LOTACAO
          FROM (SELECT 
                    VM.NRVINCULOM VINCULO,
                    PE.NRCPFPESSOA,
                    VM.NMVINCULOM NOME,
                    VH.NRESTRUTLEGAL NRLEGAL,
                    EHL.NMRAZSOCESTRUT RAZAOLEGAL,
                    EHL.CDCNPJESTRUT CNPJLEGAL,
                    SUBSTR(EHL.CDCNPJESTRUT, 0,8) CNPJRAIZ,
                    ENVIO.NRESTRUTLOTACAO,
                    LOT.CDFPAS,
                    LOT.CDTERCEIRO,
                    DECODE(ICF.NREVENTOM,   '3060', '2010', --/FGTS/
                                            '3180', '2050', --/FGTS Rescisão/
                                            '3120', '2020', --/INCID_FGTS_13SAL - FGTS Décimo Terceiro/
                                            '3210', '2050', --/FGTS 13º Rescisão/
                                            ICF.NREVENTOM) NREVENTOM,
                    CASE
                      WHEN ICF.NREVENTOM = 1310 THEN ICF.VRREFFOLHA
                      WHEN ICF.NREVENTOM = 20 AND NVL(ICF.VRREFFOLHA, 0) != 0 THEN ICF.VREVENTOFOLHA / ICF.VRREFFOLHA
                      ELSE ICF.VREVENTOFOLHA
                    END VREVENTOFOLHA,
                    ENVIO.VRBASEFGTSMENSAL,
                    ENVIO.VRBASEFGTS13,
                    VM.DTRESCISAOVINC
                  FROM GPE_VINCULOM VM
                  JOIN GPE_VINCULOH VH ON VH.NRORG = VM.NRORG
                                      AND VH.NRVINCULOM = VM.NRVINCULOM
                                      AND VH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_VINCULOH
                                                                WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                  AND NRORG      = VM.NRORG
                                                                  AND NRVINCULOM = VM.NRVINCULOM )
                  JOIN GPE_PESSOAH PE ON PE.NRPESSOA = VM.NRPESSOA 
                                    AND PE.NRORG = VM.NRORG 
                                    AND PE.DTMESCOMPETENC = (SELECT MAX(DTMESCOMPETENC) FROM GPE_PESSOAH 
                                                              WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                AND NRORG = PE.NRORG
                                                                AND NRPESSOA = PE.NRPESSOA)
                  /*ESTRUTURA LEGAL*/
                  JOIN ESTRUTURAM EML ON EML.NRORG = VH.NRORG
                                          AND EML.NRESTRUTURAM = VH.NRESTRUTLEGAL
                  LEFT JOIN ESTRUTURAH EHL ON EHL.NRORG        = VH.NRORG
                                          AND EHL.NRESTRUTURAM = EML.NRESTRUTURAM
                                          AND EHL.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = VH.NRORG
                                                                      AND NRESTRUTURAM = EML.NRESTRUTURAM )
                  JOIN ESO_ENVIOCADASTROS ENVIO ON ENVIO.NRORG = VM.NRORG
                                                    AND TRUNC(ENVIO.DTMESCOMPETENC, 'MM') = :P_COMPETENCIA
                                                    AND ENVIO.NRTPAMBIENTE = 1
                                                    AND ENVIO.IDATIVO = 'S' AND ENVIO.NRENVIOEXCL IS NULL 
                                                    /*MOVIMENTO_ESOCIAL*/
                                                    AND ENVIO.IDSTATUS IN('C','A','M')
                                                    AND (ENVIO.NRTABELA = :P_NRTABELA OR :P_NRTABELA IS NULL)
                  JOIN ESO_EVENTO EVESO ON EVESO.NREVENTO = ENVIO.NREVENTO
                                            AND EVESO.NMEVENTO IN ('S-2299','S-2399','S-1200')
                  LEFT JOIN ESTRUTURAH EMP ON EMP.NRORG        = ENVIO.NRORG
                                          AND EMP.NRESTRUTURAM = ENVIO.NRESTRUTURAM
                                          AND EMP.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = EMP.NRORG
                                                                      AND NRESTRUTURAM = EMP.NRESTRUTURAM )
                  LEFT JOIN ESTRUTURAH LOT ON LOT.NRORG        = ENVIO.NRORG
                                          AND LOT.NRESTRUTURAM = ENVIO.NRESTRUTLOTACAO
                                          AND LOT.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM ESTRUTURAH
                                                                    WHERE DTMESCOMPETENC <= :P_COMPETENCIA
                                                                      AND NRORG        = LOT.NRORG
                                                                      AND NRESTRUTURAM = LOT.NRESTRUTURAM )
                  LEFT JOIN CNAEFISCAL CF ON CF.CDCNAE = LOT.CDCNAE
                  LEFT JOIN (SELECT PEM.NRORG, PEM.NRESTRUTURAM, MAX(DECODE(ITG.NREVENTOM,3590,ITG.VRPERCENT,0)) VRPERCEMPRESA, MAX(DECODE(ITG.NREVENTOM,3870,ITG.VRPERCENT,0)) VRPERCTERCEIROS
                              FROM GPE_PARAMESTRUTM PEM
                              JOIN GPE_PARAMESTRUTH PEH ON PEH.NRORG = PEM.NRORG
                                                        AND PEH.NRPARAMESTRUTM = PEM.NRPARAMESTRUTM
                                                        AND PEH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM GPE_PARAMESTRUTH H1
                                                                                    WHERE H1.DTMESCOMPETENC <= :P_COMPETENCIA
                                                                                      AND H1.NRORG = PEH.NRORG
                                                                                      AND H1.NRPARAMESTRUTM = PEH.NRPARAMESTRUTM )
                              JOIN FPA_PARAMFOLHAM PFM ON PFM.NRORG = PEH.NRORG
                                                      AND PFM.NRPARAMFOLHAM = PEH.NRPARAMFOLHAM
                              JOIN FPA_PARAMFOLHAH PFH ON PFH.NRORG = PFM.NRORG
                                                      AND PFH.NRPARAMFOLHAM = PFM.NRPARAMFOLHAM
                                                      AND PFH.DTMESCOMPETENC = ( SELECT MAX(DTMESCOMPETENC) FROM FPA_PARAMFOLHAH H1
                                                                                  WHERE H1.DTMESCOMPETENC <= :P_COMPETENCIA
                                                                                    AND H1.NRORG = PFH.NRORG
                                                                                    AND H1.NRPARAMFOLHAM = PFH.NRPARAMFOLHAM )
                              JOIN FPA_GRPINCIDE GRI ON GRI.NRORG = PFH.NRORG
                                                    AND GRI.NRGRPINCIDE = PFH.NRGRPINCIDE
                              JOIN FPA_GRPITEM ITG ON ITG.NRORG = GRI.NRORG
                                                  AND ITG.NRGRPINCIDE = GRI.NRGRPINCIDE
                              WHERE PEM.NRORG = :P_NRORG
                              GROUP BY PEM.NRORG, PEM.NRESTRUTURAM) PER ON PER.NRORG = LOT.NRORG
                                                                      AND PER.NRESTRUTURAM = LOT.NRESTRUTURAM

                  /*CÁLCULO DE FOLHA*/
                  JOIN FPA_ITECALCFOLHA ICF ON ICF.NRORG = VM.NRORG
                                          AND ICF.NRVINCULOM = VM.NRVINCULOM
                                          AND ICF.NREVENTOM IN (3060, 3180, 3210, 3120)
                  JOIN FPA_CALCULOFOLHA CF ON CF.NRORG = ICF.NRORG
                                          AND CF.NRCALCULOFOLHA = ICF.NRCALCULOFOLHA
                                          AND ( :P_NRTABELA = 1 AND CF.NRTIPOMOVIMENT IN (1, 2, 10, 19, 22)
                                                                 OR
                                                :P_NRTABELA = 4 AND CF.NRTIPOMOVIMENT IN (4)
                                                                 OR :P_NRTABELA IS NULL AND CF.NRTIPOMOVIMENT IN (1, 2, 4, 10, 19, 22) )
                  /*MOVIMENTO_TIPO*/
                                          AND CF.NRTPMODALIDCAL = :P_MODALIDADECAL
                                          AND CF.DTOCORRENCIA = :P_COMPETENCIA
                                          AND (CF.NROCORRECAL = :P_OCORRECAL OR :P_IGNORAOCORR = 'S')

                  WHERE VM.NRORG = :P_NRORG
                    AND ( INSTR(','||:P_NRVINCULO||',', ','||VM.NRVINCULOM||',' ) > 0 OR :P_NRVINCULO IS NULL ) --[15-02-12][multisearch_vinculo_156497]
                    AND ( VM.DTADMISSAOVINC <= LAST_DAY(:P_COMPETENCIA) )
                    AND ( VM.DTRESCISAOVINC >= :P_COMPETENCIA OR VM.DTRESCISAOVINC IS NULL )
                    AND ( INSTR(','||:P_NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :P_NRESTRUTLEGAL IS NULL )
                    AND ((EVESO.NMEVENTO = 'S-1200' AND ENVIO.NRCHAVE = VM.NRPESSOA) OR (EVESO.NMEVENTO IN ('S-2299','S-2399') AND ENVIO.NRCHAVE = VM.NRVINCULOM))
                    AND SUBSTR(EMP.CDCNPJESTRUT,1,8) = SUBSTR(EHL.CDCNPJESTRUT,1,8) AND EMP.CDCNPJESTRUT IS NOT NULL
                  GROUP BY 
                    VM.NRVINCULOM,
                    PE.NRCPFPESSOA,
                    VM.NMVINCULOM,
                    VH.NRESTRUTLEGAL,
                    EHL.NMRAZSOCESTRUT,
                    EHL.CDCNPJESTRUT,
                    ENVIO.VRBASE13,
                    ENVIO.DTMESCOMPETENC,
                    SUBSTR(EHL.CDCNPJESTRUT, 0,8),
                    ENVIO.NRESTRUTLOTACAO,
                    LOT.CDFPAS,
                    LOT.CDTERCEIRO,
                    ICF.NREVENTOM,
                    ICF.VREVENTOFOLHA,
                    ICF.VRREFFOLHA,
                    ENVIO.VRBASEFGTSMENSAL,
                    ENVIO.VRBASEFGTS13,
                    VM.DTRESCISAOVINC
                    ) PIVOT ( SUM(VREVENTOFOLHA)
                                FOR NREVENTOM IN ( 2010 AS FGTS, 2020 AS FGTS_13, 2050 AS FGTS_RESCISAO ) ) PRI
      GROUP BY NOME, NRCPFPESSOA, NRLEGAL, RAZAOLEGAL, CNPJLEGAL, PRI.CDFPAS, PRI.CDTERCEIRO, CNPJRAIZ, PRI.FGTS, PRI.NRESTRUTLOTACAO, PRI.VRBASEFGTSMENSAL, PRI.VRBASEFGTS13, PRI.FGTS_RESCISAO, PRI.FGTS_13, PRI.DTRESCISAOVINC )
      ORDER BY CNPJRAIZ, CNPJLEGAL, NOME
            ";
}
