<?php

namespace HCM\PTO\Logic\AFD;

/**
 * Registro. Responsavel por gerar um conteudo (texto) de acordo com um tipo especifico de informacao
 * 
 * 
 */
 
use HCM\Util\Util;

abstract class Registro extends Manipulador implements Geravel {
    /** @var ValidadorRegistro */
    private $validadorRegistro;
    /** @var string[] */
    private $itens;
    
    public function __construct(\Doctrine\ORM\EntityManager $entityManager, Parametros $parametros, ?ValidadorRegistro $validadorRegistro = null) {
        $this->validador = $validadorRegistro;
        parent::__construct($entityManager, $parametros);
    }
    
    /**
     * Responsavel por obter o array de itens que serao as linhas do arquivo
     * @return string[]
     */
    public abstract function itens(): array;
    
    /**
     * Responsavel por obter o array de itens que foram gerados previamente
     * @return string[]
     */
    public abstract function itensUltimaGeracao(): array;
    
    /**
     * Responsavel por obter a quantidade de itens que foram gerados previamente
     * @return int
     */
    public abstract function quantidadeItensUltimaGeracao(): int;
    
    /**
     * Responsavel por gerar o texto que sera o conteudo do registro
     * @return string
     */
    public function geraConteudo(): string {
         $this->itens = $this->itens();
         $this->incluiValidacaoItens();
         return implode(Util::CRLF, $this->itens);
    }
    
    /**
     * Informa se o registro possui validador (CRC, hash)
     * @return bool
     */
    public function hasValidador(): bool {
        return !is_null($this->validador);
    }
    
    /**
     * Inclui o registro de validacao caso o registro possua validador
     * @return void
     */
    public function incluiValidacaoItens(): void {
        if (!$this->hasValidador()) {
            return;
        }
        $this->itens = $this->validador->incluiValidacaoItens($this->itens);
    }
}