<?php
namespace HCM\VAB\Services;

use HCM\VAB\Repository\BeneficioRepository;
use Doctrine\ORM\EntityManager;
use HCM\Util\DateUtil;
use \Geral\Util\Ambiente;
use \Setforms2\Shared\ORM\DateTime;
use HCM\Util\HCMParameters;
use Folha\Entities\FpaVincevento;
use Folha\Entities\FpaFaixatabela;
use Folha\Entities\FpaValorreferenciah;
use Folha\Entities\VabBeneficiario;
use HCM\Geral\Services\PessoaService;
use HCM\Util\Messages;
use HCM\Geral\Services\VinculoServices;
use HCM\Util\Repositories;
use HCM\Util\FPA;
use Zeedhi\Framework\HTTP\Kernel;
use HCM\FPA\Services\ExportacaoExcelService;
use Folha\Entities\GpePessoah;
use Folha\Entities\GpePessoa;
use Folha\Entities\GpeVinculom;
use Folha\Entities\VabBeneficiom;
class BeneficioService {

    /**
     * @var beneficioRepository
     */
    protected $beneficioRepository;
    protected $now; 
    protected $codigo;
    
    public function __construct(EntityManager $entityManager, BeneficioRepository $beneficioRepository, HCMParameters $hcmParameters,PessoaService $pessoaService, VinculoServices $vinculoServices,Kernel $httpKernel, ExportacaoExcelService $exportacaoExcelService) {
        $this->entityManager = $entityManager;
        $this->beneficioRepository = $beneficioRepository;
        $this->hcmParameters = $hcmParameters;
        $this->now = new \DateTime();  
        $this->pessoaService = $pessoaService;
        $this->vinculoServices = $vinculoServices;
        $this->vabBeneficiario = Repositories::VAB_BENEFICIARIO;
        $this->httpKernel = $httpKernel;
        $this->exportacaoExcelService = $exportacaoExcelService;
    }
    
     public function gravar_template(){
        try {
            $file = $this->makeCode();
            return $this->downloadTemplateBeneficio($file);
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    private function makeCode(){
        try {
            $file = new \PHPExcel();
            $file->setActiveSheetIndex(0);
            $activeSheet = $file->getActiveSheet();
            
            $activeSheet->setCellValue('A1', 'Número Benefício');
            $activeSheet->getStyle('A1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('A')->setAutoSize(true);
            
            $activeSheet->setCellValue('B1', 'Número de Inscrição');
            $activeSheet->getStyle('B1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('B')->setAutoSize(true);
            
            $activeSheet->setCellValue('C1', 'Data Inscrição');
            $activeSheet->getStyle('C1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('C')->setAutoSize(true);
            
            $activeSheet->setCellValue('D1', 'Dt Cancelamento');
            $activeSheet->getStyle('D1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('D')->setAutoSize(true);
            
            $activeSheet->setCellValue('E1', '% Subsídio');
            $activeSheet->getStyle('E1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('E')->setAutoSize(true);
            
            $activeSheet->setCellValue('F1', 'Nr. Vínculo');
            $activeSheet->getStyle('F1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('F')->setAutoSize(true);
            
            $activeSheet->setCellValue('G1', 'Matrícula');
            $activeSheet->getStyle('G1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('G')->setAutoSize(true);
            
            $activeSheet->setCellValue('H1', 'Matrícula eSocial');
            $activeSheet->getStyle('H1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('H')->setAutoSize(true);
            
            $activeSheet->setCellValue('I1', 'Vínculo');
            $activeSheet->getStyle('I1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('I')->setAutoSize(true);
            
            $activeSheet->setCellValue('J1', 'Nome do Dependente (caso seja dependente)');
            $activeSheet->getStyle('J1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('J')->setAutoSize(true);
            
            $activeSheet->setCellValue('K1', 'Tipo Beneficiário');
            $activeSheet->getStyle('K1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('K')->setAutoSize(true);
            
            $activeSheet->setCellValue('L1', 'Qtd. Benefício');
            $activeSheet->getStyle('L1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('L')->setAutoSize(true);
            
            $activeSheet->setCellValue('M1', 'Observação');
            $activeSheet->getStyle('M1')->getBorders()->getOutline()->setBorderStyle(\PHPExcel_Style_Border::BORDER_THIN);
            $activeSheet->getColumnDimension('M')->setAutoSize(true);
            return $file;
        } catch (\Exception $e) {
            throw $e;
        }
    }
    
    public function getTipoBeneficioGerencialVigente($row){
        try{
            $data = null;
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $nrestruturageren = $row['NRESTRUTGEREN'];
            $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
            
            $data = $this->beneficioRepository->getTipoBeneficioGerencialVigente($nrorg, $nrestruturageren, $dtmescompetenc); 
            
            if(!empty($data)){
                return $data;
            }
            return $data;
        }catch(Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function verificaTipoBeneficioGerencialExistente($row){
        try{
            $msg = null;
            $msg = $this->getTipoBeneficioGerencialExistente($row); 
            
            if(!empty($msg)){
                return $msg;
            }
            return $msg;
        }catch(Exception $e){
            $this->entityManager->rollback();
            throw $e;
        }
    }
    
    public function getTipoBeneficioGerencialExistente($row){
        $message = null;
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $nrtipobeneficio = $row['NRTIPOBENEFICIO'];
        $nrestruturageren = $row['NRESTRUTGEREN'];
        $nrtpbeneficiogeren = !empty($row['NRTPBENEFICIOGERENCIAL']) ? $row['NRTPBENEFICIOGERENCIAL'] : null ;
        
        $existe = $this->beneficioRepository->getTipoBeneficioGerencialExistente($nrorg, $nrtipobeneficio, $nrestruturageren,$nrtpbeneficiogeren);
        
        if(!empty($existe)){
            return $message = "Operação Bloqueada!! <br><br>O tipo de benefício já está vínculado a estrutura gerencial escolhida! Favor utilizar o registro anterior ou remove-lo.";
        }
        
        return $message;
    }
    
    private function downloadTemplateBeneficio($file) {
        try {
            $nomeArquivo = "Formulario_Beneficio.xls";
            $expDir = FPA::getTempDirectory();
            $write = \PHPExcel_IOFactory::createWriter($file, 'Excel5');
            $write->save($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);
            
            $file = ($expDir . DIRECTORY_SEPARATOR . $nomeArquivo);
            return $file;
            
        } catch (\Exception $e) {
            throw new \Exception("Erro ao realizar a exportação");
        }

    }
    public function importarPlanilhaBeneficio($row) {
    $nrorg       = $this->hcmParameters->getNrorgTrabalho();
    $nrorgPadrao = $this->hcmParameters->getNrorgPadrao();
    $messages    = array(); 
    $dados = $this->getDadosPlanilha($row['DSPATH'][0]);
    if($dados == null){
        return array("msg"=>"O número benefício não pode ser vazio.");
    } 
    foreach($dados as $lin => $dado) {
        $res = $this->insereDadosPlanilha($lin + 2, $nrorg, $nrorgPadrao,
            $dado['NRBENEFICIO'], $dado['MATRICULA'],$dado['MATRICULAESOCIAL'],$dado['NRISCRICAO'],
            $dado['DTISCRICAO'],$dado['DTCANCELAMENTO'], $dado['PERCENTSUBSIDIO'], 
            $dado['NRVINCULO'],$dado['VINCULO'], $dado['NAMEDEPENDENTE'],
            $dado['TIPOBENEFICIOARIO'],$dado['QTDBENEFICIO'],
            $dado['OBS']);
        if(!empty($res))
            $messages[] = $res;
    }
    return $messages;
    }

    public function getDadosPlanilha($path) {
        $head = array(
            'NRBENEFICIO',
            'NRISCRICAO',
            'DTISCRICAO',
            'DTCANCELAMENTO',
            'PERCENTSUBSIDIO',
            'NRVINCULO',
            'MATRICULA',
            'MATRICULAESOCIAL',
            'VINCULO',
            'NAMEDEPENDENTE',
            'TIPOBENEFICIOARIO',
            'QTDBENEFICIO',
            'OBS'
            );
        //Colunas de data
        $dateColumns = array('DTISCRICAO', 'DTCANCELAMENTO');
        $file = $this->httpKernel->factoryFromFileData($path, FPA::getTempDirectory()."/");
        $caminhoNovo = DIRECTORY_SEPARATOR . $file->getPathname();
        $file = \PHPExcel_IOFactory::load($caminhoNovo);
        $file->setActiveSheetIndex(0);
        return $this->exportacaoExcelService->preparaInfo_generic($file, $head,'NRBENEFICIO', $dateColumns);
    }
    
    public function insereDadosPlanilha($lin, $nrorg, $nrorgPadrao,$nrbeneficio, $matricula, $matriculaesocial, $nriscricao, $dtiscricao,
            $dtcancelamento, $percentsubsidio, $nrvinculom, $vinculo, $namedependente, $tipobeneficiario, 
            $qtdbeneficio, $obs){
        $dtmescompetenc = $this->hcmParameters->getDtCompetencia();
        if($dtiscricao)
            $dtiscricao = DateUtil::getDataDeString($dtiscricao, DateUtil::FORMATO_BRASILEIRO, true);
        else
            $dtiscricao = null;
        if($dtcancelamento)
            $dtcancelamento = DateUtil::getDataDeString($dtcancelamento, DateUtil::FORMATO_BRASILEIRO, true);
        else
            $dtcancelamento = null;
        $tipobeneficiario = strtoupper($tipobeneficiario);
        
        if($nrvinculom == null){
            if($matricula){
                $objVinculom=GpeVinculom::findOneByNrorgAndCdmatricula($nrorg, $matricula);
                if ($objVinculom == null)
                    return array("msg"=>"O código da matrícula do vínculo não foi encontrado na linha $lin.");
                else
                    $nrvinculom = $objVinculom->getNrvinculom();
            }else{
                if($matriculaesocial){
                    $objVinculom = $this->entityManager->getRepository(Repositories::GPE_VINCULOH)->retornaVinculoPorMatriculaESocial($nrorg, $nrvinculom, $matriculaesocial, DateUtil::getDataDeString($dtmescompetenc, DateUtil::FORMATO_BRASILEIRO, true), null);
                    if ($objVinculom == null)
                        return array("msg"=>"O código da matrícula do eSocial não foi encontrado na linha $lin.");
                    else
                        $nrvinculom = $objVinculom->getNrvinculom();
                }else{
                    return array("msg"=>"O código do vínculo, da matrícula do vínculo ou da matrícula do eSocial não foram encontrados. É necessário o preenchimento de pelo menos um desses campos.");
                }
            }
        }
        if($tipobeneficiario == "TITULAR")
            $tipobeneficiario = "T";
        if($tipobeneficiario == "DEPENDENTE")
            $tipobeneficiario = "D";
        if($dtiscricao == null)
            return array("msg"=>"Data de inscrição inválida na linha $lin.");
        if($tipobeneficiario != "T" && $tipobeneficiario != "D")
            return array("msg"=>"Erro na linha $lin, tipo de beneficiário deve ser preenchido como titular ou dependente.");
        $beneficio = VabBeneficiom::findOneByNrorgAndNrbeneficiom($nrorg, $nrbeneficio);
        if($beneficio == null)
            return array("msg"=>"Número de benefício $beneficio não encontrado na linha $lin.");
        $namedependente = strtoupper($namedependente);
        if($qtdbeneficio == null)
            return array("msg"=>"A quantidade de benefício deve ser informada na linha $lin.");
        if(!is_numeric($qtdbeneficio))
            return array("msg"=>"A quantidade de benefício deve ser numérica na linha $lin.");
        if($tipobeneficiario=="D"){
            $pessoa = GpePessoah::findOneByNrorgAndNmpessoa($nrorg, $namedependente);
            if ($pessoa == null)
                return array("msg"=>"Cadastro do dependente $pessoa na linha $lin, não encontrado no sistema.");
            $nrpessoa = $pessoa->getNrpessoa();
        }else{
            $vinculo=GpeVinculom::findOneByNrorgAndNrvinculom($nrorg, $nrvinculom);
            if ($vinculo == null)
                return array("msg"=>"Número do vinculo $nrvinculom na linha $lin, não encontrado no sistema.");
            $nrpessoa = $vinculo->getNrpessoa();
        }
            
        $codigo = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIARIO', 1, 12, true);
        $obj = new \Folha\Entities\VabBeneficiario();
        $obj->setNrbeneficiario($codigo);
        $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtinclusao(DateUtil::getDataAtual());
        $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $obj->setDtultatu(DateUtil::getDataAtual());
        $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
        $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        $obj->setIdativo('S');
        $obj->setNrbeneficiom($nrbeneficio);
        $obj->setNrinscbeneficiario($nriscricao);
        $obj->setDtinscbeneficiario($dtiscricao);
        $obj->setDtcancbeneficiario($dtcancelamento);
        $obj->setVrpercsubsidio($percentsubsidio);
        $obj->setNrvinculom($nrvinculom);
        $obj->setNrpessoa($nrpessoa);
        $obj->setIdtpbeneficiario($tipobeneficiario);
        $obj->setQtbeneficio($qtdbeneficio);
        $obj->setDsobsbeneficiario($obs);
        $this->entityManager->persist($obj);
        $this->entityManager->flush();
        return null;
    }
    
    public function getBeneficiarioPorEvento($parameters, $conditions) {
        return $this->beneficioRepository->getBeneficiarioPorEvento($parameters, $conditions);
    }
    
    public function getBeneficiario($parameters, $conditions) {
        return $this->beneficioRepository->getBeneficiario($parameters, $conditions);
    }
    
    public function getBeneficioPorVinculo($mesCompetencia, $nrorg, $idtpbeneficiario, $nrvinculom, $nrpessoadep, $nrorgpadrao, $filtro){
        return $this->beneficioRepository->getBeneficioPorVinculo($mesCompetencia, $nrorg, $idtpbeneficiario, $nrvinculom, $nrpessoadep, $nrorgpadrao, $filtro);
    }
    
    public function retornaBefeficiarioVinculo($nrorg, $nrorgpadrao, $nrvinculom, $mescompetencia, $idtpbeneficiario, $idativo, $filtro){
        return $this->beneficioRepository->retornaBefeficiarioVinculo($nrorg, $nrorgpadrao, $nrvinculom, $mescompetencia, $idtpbeneficiario, $idativo, $filtro);
    }
    
    public function novoMestre($dtiniciovigencia, $dtfinalvigencia, $idativo)
    {
        $vab_beneficiom = Repositories::VAB_BENEFICIOM;
        
        $novo = new $vab_beneficiom;
        $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $novo->setDtiniciovigencia($dtiniciovigencia);
        $novo->setDtfinalvigencia($dtfinalvigencia);
        $novo->setIdativo($idativo);  
        $novo->setDtinclusao(DateUtil::getDataAtual()); 
        $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $novo->setDtultatu(DateUtil::getDataAtual());
        $novo->setCdoperultatu($this->hcmParameters->getCdoperador());
        $novo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $novo->setNrbeneficiom(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIOM', 1, 12, true));
        
        $this->entityManager->persist($novo);
        $this->entityManager->flush();
        
        return $novo;
    }
    
    public function novoHistorico(
        $nrbeneficiom, $nrtpbeneficio, $nrformabeneficio, $nrestruturam, $dtmescompetenc, $dtreferencia, $dsbeneficio, $tpperiodobenef, $dsobsbeneficio, $nrtabelafolhapri, $nrtabelafolhasec, $idativo, $nreventomensa, 
        $nrevetitularemp, $nrevedepempresa, $nrevetitularvinc, $nrevedepevinc, $nrtabfaixasaltit, $nrtabfaixasaldep, $nrmesaniversario, $nrdiavencboleto, $vrlinearbeneficio, $nridademaxsubfilhos, $nridademaxsubdep, 
        $nrtipoparticipatitular, $vrparticipatitular, $nrtipoparticipadep, $vrparticipadepe, $vrlinearbeneficiodep, $nreventomensadep, $vrpisotitular, $vrpisodep, $vrtetodep, $vrtetotitular, $sexosubsidiofilho, 
        $sexosubsidiodep, $idconsfaltabenef, $qtmesesfaltabenef, $idconsatestbenef, $qtmesesatestbenef, $idconsafastbenef, $qtmesesafastbenef, $idconsferiasbenef, $qtmesesferiasbenef, $idconsferiadobenef, 
        $qtmesesferiadobenef, $idconsdsrbenef, $qtmesesdsrbenef, $idgerabeneffalta, $idgerabenefatest, $idgerabenefafast, $idgerabenefferias, $consvincresmesant, $nrestrutsind, $dtdisponib, $idpropadmis, 
        $idpropresc, $nreventocopart, $nreventocopartdep, $percoparticipatitu, $percoparticipadepe, $vrdepsind, $vrtitularsind, $idpropdtinscr, $idpropdtcanc, $idduplicmens, $nrdiarescisao, $nreventotaxa, 
        $vreventotaxa, $vrtetotitularsind, $vrpisotitularsind, $vrpisodepsind, $vrtetodepsind, $vrtetotitularmens, $vrpisotitularmens, $idgeradescafast, $periodicidademes, $idgerbenefadmissao, $idgerbenefrescisao)
    {
        $vab_beneficioh = Repositories::VAB_BENEFICIOH;
        
        $novo = new $vab_beneficioh;
        $novo->setNrorg($this->hcmParameters->getNrorgTrabalho());
        $novo->setNrbeneficiom($nrbeneficiom);
        $novo->setNrtpbeneficio($nrtpbeneficio);
        $novo->setNrformabeneficio($nrformabeneficio);
        $novo->setNrestruturam($nrestruturam);
        $novo->setDtmescompetenc($dtmescompetenc);
        $novo->setDtreferencia($dtreferencia);
        $novo->setDsbeneficio($dsbeneficio);
        $novo->setTpperiodobenef($tpperiodobenef);
        $novo->setDsobsbeneficio($dsobsbeneficio);
        $novo->setNrtabelafolhapri($nrtabelafolhapri);
        $novo->setNrtabelafolhasec($nrtabelafolhasec);
        $novo->setIdativo($idativo);
        $novo->setNreventomensa($nreventomensa);
        $novo->setNrevetitularemp($nrevetitularemp);
        $novo->setNrevedepempresa($nrevedepempresa);
        $novo->setNrevetitularvinc($nrevetitularvinc);
        $novo->setNrevedepevinc($nrevedepevinc);
        $novo->setNrtabfaixasaltit($nrtabfaixasaltit);
        $novo->setNrtabfaixasaldep($nrtabfaixasaldep);
        $novo->setNrmesaniversario($nrmesaniversario);
        $novo->setNrdiavencboleto($nrdiavencboleto);
        $novo->setVrlinearbeneficio($vrlinearbeneficio);
        $novo->setNridademaxsubfilhos($nridademaxsubfilhos);
        $novo->setNridademaxsubdep($nridademaxsubdep);
        $novo->setNrtipoparticipatitular($nrtipoparticipatitular);
        $novo->setVrparticipatitular($vrparticipatitular);
        $novo->setNrtipoparticipadep($nrtipoparticipadep);
        $novo->setVrparticipadepe($vrparticipadepe);
        $novo->setVrlinearbeneficiodep($vrlinearbeneficiodep);
        $novo->setNreventomensadep($nreventomensadep);
        $novo->setVrpisotitular($vrpisotitular);
        $novo->setVrpisodep($vrpisodep);
        $novo->setVrtetodep($vrtetodep);
        $novo->setVrtetotitular($vrtetotitular);
        $novo->setSexosubsidiofilho($sexosubsidiofilho);
        $novo->setSexosubsidiodep($sexosubsidiodep);
        $novo->setIdconsfaltabenef($idconsfaltabenef);
        $novo->setQtmesesfaltabenef($qtmesesfaltabenef);
        $novo->setIdconsatestbenef($idconsatestbenef);
        $novo->setQtmesesatestbenef($qtmesesatestbenef);
        $novo->setIdconsafastbenef($idconsafastbenef);
        $novo->setQtmesesafastbenef($qtmesesafastbenef);
        $novo->setIdconsferiasbenef($idconsferiasbenef);
        $novo->setQtmesesferiasbenef($qtmesesferiasbenef);
        $novo->setIdconsferiadobenef($idconsferiadobenef);
        $novo->setQtmesesferiadobenef($qtmesesferiadobenef);
        $novo->setIdconsdsrbenef($idconsdsrbenef);
        $novo->setQtmesesdsrbenef($qtmesesdsrbenef);
        $novo->setIdgerabeneffalta($idgerabeneffalta);
        $novo->setIdgerabenefatest($idgerabenefatest);
        $novo->setIdgerabenefafast($idgerabenefafast);
        $novo->setIdgerabenefferias($idgerabenefferias);
        $novo->setConsvincresmesant($consvincresmesant);
        $novo->setNrestrutsind($nrestrutsind);
        $novo->setDtdisponib($dtdisponib);
        $novo->setIdpropadmis($idpropadmis);
        $novo->setIdpropresc($idpropresc);
        $novo->setNreventocopart($nreventocopart);
        $novo->setNreventocopartdep($nreventocopartdep);
        $novo->setPercoparticipatitu($percoparticipatitu);
        $novo->setPercoparticipadepe($percoparticipadepe);
        $novo->setVrdepsind($vrdepsind);
        $novo->setVrtitularsind($vrtitularsind);
        $novo->setIdpropdtinscr($idpropdtinscr);
        $novo->setIdpropdtcanc($idpropdtcanc);
        $novo->setIdduplicmens($idduplicmens);
        $novo->setNrdiarescisao($nrdiarescisao);
        $novo->setNreventotaxa($nreventotaxa);
        $novo->setVreventotaxa($vreventotaxa);
        $novo->setVrtetotitularsind($vrtetotitularsind);
        $novo->setVrpisotitularsind($vrpisotitularsind);
        $novo->setVrpisodepsind($vrpisodepsind);
        $novo->setVrtetodepsind($vrtetodepsind);
        $novo->setVrtetotitularmens($vrtetotitularmens);
        $novo->setVrpisotitularmens($vrpisotitularmens);
        $novo->setIdgeradescafast($idgeradescafast);
        $novo->setPeriodicidademes($periodicidademes);
        $novo->setIdgerbenefadmissao($idgerbenefadmissao);
        $novo->setIdgerbenefrescisao($idgerbenefrescisao);
        $novo->setDtinclusao(DateUtil::getDataAtual()); 
        $novo->setCdoperinclusao($this->hcmParameters->getCdoperador());
        $novo->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
        $novo->setDtultatu(DateUtil::getDataAtual());
        $novo->setCdoperultatu($this->hcmParameters->getCdoperador());
        $novo->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
        
        $novo->setNrbeneficioh(\HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIOH', 1, 12, true));
        
        $this->entityManager->persist($novo);
        $this->entityManager->flush();
        
        return $novo;
    }
    
    public function insereDtCancelamentoDependentes($nrvinculom, $nrbeneficiom, $dtcancbeneficiario){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        
        $params = array(
                'NRORG'          => $nrorg,
                'NRVINCULOM' => $nrvinculom,
                'NRBENEFICIOM'   => $nrbeneficiom
    		);
        
        $where = "NRORG = :NRORG
                  AND NRBENEFICIOM = :NRBENEFICIOM
                  AND NRVINCULOM = :NRVINCULOM
                  AND IDTPBENEFICIARIO = 'D'
                  AND DTCANCBENEFICIARIO IS NULL";

        $beneficiarios = VabBeneficiario::getObjectsByWhere($where, $params);
        
        if($beneficiarios){
            foreach($beneficiarios as $beneficiario){
                if(!$beneficiario->getDtcancbeneficiario()){
                    $beneficiario->setDtcancbeneficiario(DateUtil::getDataDeString($dtcancbeneficiario));
                    $this->entityManager->flush();
                }
            }
        }
    }

    public function removerBeneficiarios($lista_nrpessoas,$nrestrutsind,$nrestrutlegal, 
        $nrestrutgeren,$nrbeneficiom,$nrorg,$lista_nrpessoasvinc,$dtmescompetenc,
        $nrorgpadrao,$nrvinculom_list,$tipobeneficiario,$nrestruturam,$nreventom_list) {
        $pessoasdelete = array(); 
        
        if($nrestrutsind === 'A') $nrestrutsind = null;
        else $nrestrutsind = implode($nrestrutsind, ",");
        
        if($nrestrutlegal === 'A') $nrestrutlegal = null;
        else $nrestrutlegal = implode($nrestrutlegal, ",");
        
        if($nrestrutgeren === 'A') $nrestrutgeren = null;

        if($tipobeneficiario === 'A'){
            $wheretipobeneficiario = '';
            
        }else{
            $wheretipobeneficiario = "AND IDTPBENEFICIARIO = '$tipobeneficiario'";
        }

        $wherebeneficiarios = "AND (INSTR(:LISTA_NRPESSOA, '_'||NRPESSOA||'_') > 0)";
        
        if($nrvinculom_list === 'A'){
            $lista_nrpessoas = array();
            
            $wherebeneficiarios = " $wheretipobeneficiario
            AND NRVINCULOM IN 
            (SELECT NRVINCULOM FROM GPE_VINCULOH VH WHERE
                ( INSTR(','||:NRESTRUTSIND||',', ','||VH.NRESTRUTSIND||',' ) > 0 OR :NRESTRUTSIND IS NULL )
                AND ( INSTR(','||:NRESTRUTLEGAL||',', ','||VH.NRESTRUTLEGAL||',' ) > 0 OR :NRESTRUTLEGAL IS NULL )
                AND ( VH.NRESTRUTGEREN = :NRESTRUTGEREN OR :NRESTRUTGEREN IS NULL ) 
                AND VH.NRORG = :NRORG  
                AND VH.DTMESCOMPETENC = (SELECT MAX(GVH.DTMESCOMPETENC)
                                            FROM GPE_VINCULOH GVH
                                           WHERE GVH.NRORG          = VH.NRORG
                                             AND GVH.NRVINCULOM     = VH.NRVINCULOM
                                             AND GVH.DTMESCOMPETENC <= :DTMESCOMPETENC))";
        }
        else if($lista_nrpessoas === 'A'){
            $lista_nrpessoas = $this->filtrarVinculoDependentePorVinculo($nrorg,$dtmescompetenc,$nrorgpadrao,$lista_nrpessoasvinc,$tipobeneficiario);
        }
        else{
            $lista_nrpessoas = $lista_nrpessoasvinc;
        }
        
        foreach ($lista_nrpessoas as $rowpessoas) {
            array_push($pessoasdelete,$rowpessoas['NRPESSOA']);
        }
        
        $pessoasdelete = is_array($pessoasdelete) ? "_".implode('_',array_values($pessoasdelete))."_" : $pessoasdelete;
    
        $params = array(
                'NRORG'          => $nrorg,
                'LISTA_NRPESSOA' => $pessoasdelete,
                'NRBENEFICIOM'   => $nrbeneficiom,
                'DTMESCOMPETENC' => $dtmescompetenc,
                'NRESTRUTSIND'   => $nrestrutsind,
                'NRESTRUTGEREN'  => $nrestrutgeren,
                'NRESTRUTLEGAL'  => $nrestrutlegal,
    		);
    		
    	if($nreventom_list) $nreventom_list = " AND NREVENTOM IN(" . implode(", ", $nreventom_list) . ")";
    	else $nreventom_list = "";
    	
    	$where = "NRORG = :NRORG
                  $nreventom_list
                  AND NRBENEFICIOM = :NRBENEFICIOM
                  $wherebeneficiarios";

        $beneficiarios = VabBeneficiario::getObjectsByWhere($where, $params);

        if(empty($beneficiarios)){
            throw new \Exception(Messages::HCM02000_VAB_BENEFICIARIO_N_ENCONTR_ERROR,Messages::ERROR_MESSAGE_CODE);
        }
        
        foreach($beneficiarios as $beneficiario){
            $nreventom = $beneficiario->getNreventom();
            $nrvinculom = $beneficiario->getNrvinculom();
            $vrbeneficio = $beneficiario->getVrbeneficio();
            $qtbeneficio = $beneficiario->getQtbeneficio();
            $tipovalor = $beneficiario->getIdvalordifebenef();
            $vincevento = $beneficiario->getNrvincevento();
            $beneficiario->delete();
            $this->entityManager->flush();
            /*--------------- Rotina para associação vínculo evento--
            if($vincevento){
                if($tipovalor === 'REFERENCIA'){
                    $this->subtrairVrRefFixoVincEvento($nreventom,$nrestruturam,$nrvinculom,$qtbeneficio);
                }else{
                    $this->subtrairVrEventoFixoVincEvento($nreventom,$nrestruturam,$nrvinculom,$vrbeneficio,$qtbeneficio);
                }
            }-------------------------------------------------------*/
        }
    }
    
    
    public function inserirVincEvento($parameters) {
        try {
            $percent = $parameters['IDVALORDIFEBENEF'] === 'REFERENCIA' ? 100 : 0;
            $vlrMultiplicador = $parameters['IDVALORDIFEBENEF'] === 'REFERENCIA' ? $parameters['QTREFERENCIA'] : 1;
            $dtmescompetenc = DateUtil::getDataDeString($parameters['DTMESCOMPETENC'], DateUtil::FORMATO_BRASILEIRO, true);
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $codigo = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'FPA_VINCEVENTO', 1, 12, true);
            $obj = new \Folha\Entities\FpaVincevento();
            $obj->setNrorg($nrorg);
            $obj->setDtinclusao(DateUtil::getDataAtual()); 
            $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
            $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
            $obj->setDtultatu(DateUtil::getDataAtual());
            $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
            $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
            $obj->setVrmultiplic($vlrMultiplicador);
            $obj->setNreventom($parameters['NREVENTOM']);
            $obj->setNrvincevento($codigo);
            $obj->setNrvalorreferenciam($parameters['NRVALORREFERENCIAM']);
            $obj->setNrvinculom($parameters['NRVINCULOM']);
            $obj->setDtinivigencia($dtmescompetenc);
            $obj->setPeriodicidademes('SSSSSSSSSSSS');
            $obj->setIdtipomovimento(1);
            $obj->setVrpercent($percent);
            $obj->setVreventofixo($parameters['VRBENEFICIO']);
            $obj->setVrreffixo($parameters['QTREFERENCIA']);
            $obj->setNrestruturam($parameters['NRESTRUTURAM']);
            $this->entityManager->persist($obj);
            $this->entityManager->flush();
        } catch (\Exception $e) {
            throw $e;
        }
        return $codigo;
    }

    public function cadastroAutomatico($lista_nrpessoas,$nrestrutsind,$nrestrutlegal, 
        $nrestrutgeren,$nrbeneficiom,$nrorg,$lista_nrpessoasvinc,$dtmescompetenc,
        $nrorgpadrao,$nrvinculom_list,$nrtabelafolhapri,$vrbeneficio,$qtbeneficio,
        $nreventom_list,$nrestruturam,$nrtabelafolhasec,$tpperiodobenef,$nrreferenciam,
        $tipobeneficiario,$tipovalor,$dtinscbeneficiario)
    {
        try{
            $observacao = null;
            if($nrestrutgeren === 'A'){
                $nrestrutgeren = null;
            }
            if($nrvinculom_list === 'A'){
                $lista_nrpessoas = [];
                $pilha_nrpessoasPorEstrutLegal = [];
                
                if($nrestrutsind === 'A'){
                    if($nrestrutlegal === 'A')
                        array_push( $pilha_nrpessoasPorEstrutLegal, $this->filtrarVinculoDependentePorEstrutura($nrorg,$dtmescompetenc,null,null,$nrestrutgeren,$nrorgpadrao,$lista_nrpessoasvinc,$tipobeneficiario) );
                    else{
                        foreach($nrestrutlegal as $nrestrutleg){
                            array_push( $pilha_nrpessoasPorEstrutLegal, $this->filtrarVinculoDependentePorEstrutura($nrorg,$dtmescompetenc,null,$nrestrutleg,$nrestrutgeren,$nrorgpadrao,$lista_nrpessoasvinc,$tipobeneficiario) );
                        }
                    }
                }else{
                    foreach($nrestrutsind as $nrestrutsin){
                        if($nrestrutlegal === 'A')
                            array_push( $pilha_nrpessoasPorEstrutLegal, $this->filtrarVinculoDependentePorEstrutura($nrorg,$dtmescompetenc,$nrestrutsin,null,$nrestrutgeren,$nrorgpadrao,$lista_nrpessoasvinc,$tipobeneficiario) );
                        else{
                            foreach($nrestrutlegal as $nrestrutleg){
                                array_push( $pilha_nrpessoasPorEstrutLegal, $this->filtrarVinculoDependentePorEstrutura($nrorg,$dtmescompetenc,$nrestrutsin,$nrestrutleg,$nrestrutgeren,$nrorgpadrao,$lista_nrpessoasvinc,$tipobeneficiario) );
                            }
                        }
                    }
                }

                foreach($pilha_nrpessoasPorEstrutLegal as $listas){
                    $lista_nrpessoas = array_merge( $lista_nrpessoas, $listas );
                }

            }
            else if($lista_nrpessoas === 'A'){
                $lista_nrpessoas = $this->filtrarVinculoDependentePorVinculo($nrorg,$dtmescompetenc,$nrorgpadrao,$lista_nrpessoasvinc,$tipobeneficiario);
            }else{
                $lista_nrpessoas = $lista_nrpessoasvinc;
            }
            if($tipovalor === 'TABELA'){
                if($nrtabelafolhapri){
                    $tabelafolhapri = FpaFaixatabela::findByNrtabelafolha($nrtabelafolhapri);
                }
                if($nrtabelafolhasec){
                    $tabelafolhasec = FpaFaixatabela::findByNrtabelafolha($nrtabelafolhasec);
                }
            }else if($tipovalor === 'REFERENCIA'){
                $vrbeneficio = null;
            }
            set_time_limit(0);
            $persistir = true;
            $beneficiariosCadastrados = '';
            $referenciainvalidas = '';
            $beneficiariosSemCPF = '';
            
            if(!$nreventom_list)
                $nreventom_list = ["semEvento"];
            
            foreach ($nreventom_list as $nreventom) {
                
                if($nreventom === "semEvento") $nreventom = null;
                
                foreach($lista_nrpessoas as $rows){
                    
                    $persistir = true;
                    
                    $beneficiarios = VabBeneficiario::findByNrbeneficiomAndNrpessoaAndNreventomAndNrorgAndNrvinculom($nrbeneficiom,$rows['NRPESSOA'],$nreventom,$nrorg,$rows['NRVINCULOM']);
                    if(isset($beneficiarios[0])){
                        $persistir = false;
                        $beneficiariosCadastrados .= $rows['NOMEPESSOA']."<br/>";  
                    }
                    
                    if($tipovalor === 'TABELA'){
                        $tabela = '';
                        if($rows['TIPOPESSOA'] === 'T'){
                            $tabela = $tabelafolhapri;
                        }else if($rows['TIPOPESSOA'] === 'D'){
                            $tabela = $tabelafolhasec;
                        }
                        if(empty($rows['DTNASCIMENTO'])){
                            throw new \Exception(
                                Messages::evaluateMessage(Messages::HCM00000_GER_DTNASC_PESS_N_INF_ERROR, $rows['NRPESSOA'].' - '.$rows['NOMEPESSOA']),
                                Messages::ERROR_MESSAGE_CODE
                            );
                        }
                        $vrbeneficio = $this->calculavrtabela($rows['DTNASCIMENTO'],$tabela);
                        if(empty($vrbeneficio)){
                            throw new \Exception(Messages::HCM02000_VAB_BENEFICIO_FX_ETARIA_N_ENCONT_ERROR,Messages::ERROR_MESSAGE_CODE);
                        }
                    }
                    
                    /*--------------- Rotina para associação vínculo evento--
                    if($tpperiodobenef !== 'DIARIO'){
                        $parametros = array('QTBENEFICIO' => $qtbeneficio,'NREVENTOM' => $nreventom,'NRVINCULOM' => $rows['NRVINCULOM'],'NRESTRUTURAM' => $nrestruturam,'VRBENEFICIO' =>$vrbeneficio,'NRVALORREFERENCIAM' => $nrreferenciam, 'IDVALORDIFEBENEF' => $tipovalor, 'DTMESCOMPETENC'=>$dtmescompetenc);
                        if($tipovalor === 'REFERENCIA'){
                            $nrvincevento = $this->salvarSomarVrRefFixoVincEvento($nreventom,$nrestruturam,$rows['NRVINCULOM'],$parametros);
                        }else{      
                            $nrvincevento = $this->salvarSomarVrEventoFixoVincEvento($nreventom,$nrestruturam,$rows['NRVINCULOM'],$vrbeneficio, null, $parametros);
                        }
                    }else{
                        $nrvincevento = null;
                    }-------------------------------------------------------*/
                    $nrvincevento = null;
                    


                    
                    if($nrvincevento === '!REFERENCIA'){
                        $persistir = false;
                        $referenciainvalidas .= $rows['NOMEPESSOA']."<br/>";  
                    }
                    
                    $pessoaBeneficiario = $this->entityManager->getRepository(Repositories::GPE_PESSOAH)->retornaHistoricoAtualPorPessoa($this->hcmParameters->getNrorgTrabalho(), $rows['NRPESSOA'], $this->hcmParameters->getDtCompetencia());
                    if(!$pessoaBeneficiario->getNrcpfpessoa()){
                        $persistir = false;
                        $beneficiariosSemCPF .= $rows['NOMEPESSOA']."<br/>";  
                    }    

                    
                    if($persistir){
                        $codigo = \HCM\Util\NovoCodigo::geraCodigo($this->hcmParameters->getNrorgTrabalho(), 'VAB_BENEFICIARIO', 1, 12, true);
                        $obj = new \Folha\Entities\VabBeneficiario();
                        $obj->setNrorg($this->hcmParameters->getNrorgTrabalho());
                        $obj->setDtinclusao(DateUtil::getDataAtual());
                        $obj->setCdoperinclusao($this->hcmParameters->getCdoperador());
                        $obj->setNrorginclusao($this->hcmParameters->getNrorgTrabalho());
                        $obj->setDtultatu(DateUtil::getDataAtual());
                        $obj->setCdoperultatu($this->hcmParameters->getCdoperador());
                        $obj->setNrorgultatu($this->hcmParameters->getNrorgTrabalho());
                        $obj->setIdativo('S');
                        $obj->setNrvinculom($rows['NRVINCULOM']);
                        $obj->setNrpessoa($rows['NRPESSOA']);
                        $obj->setIdtpbeneficiario($rows['TIPOPESSOA']);
                        $obj->setNreventom($nreventom);
                        $obj->setNrvincevento($nrvincevento);
                        $obj->setNrvalorreferenciam($nrreferenciam);
                        $obj->setIdvalordifebenef($tipovalor);
                        $obj->setNrbeneficiario($codigo);
                        $obj->setNrbeneficiom($nrbeneficiom);
                        $obj->setDtinscbeneficiario(DateUtil::getDataDeString($dtinscbeneficiario, DateUtil::FORMATO_BRASILEIRO, true));
                        $obj->setQtbeneficio($qtbeneficio);
                        $obj->setVrbeneficio($vrbeneficio);
                        $obj->setDsobsbeneficiario($observacao);
                        $this->entityManager->persist($obj);
                        $this->entityManager->flush();
                    }
                }
            }
            $beneficiariosInvalidos = array('CADASTRADOS' => $beneficiariosCadastrados, 'REFERENCIA' => $referenciainvalidas, 'SEMCPF'=>$beneficiariosSemCPF);
            return $beneficiariosInvalidos;
        }catch(\Exception $e){
            throw $e;
        }
    }
    
    public function filtrarVinculoDependentePorEstrutura($nrorg,$dtmescompetenc,$nrestrutsind,$nrestrutlegal,$nrestrutgeren,$nrorgpadrao,$lista_nrpessoasvinc,$tipobeneficiario){
        $resultado = $this->vinculoServices->retornaVinculosCompetencia($nrorg,$dtmescompetenc,true,$nrestrutlegal,$nrestrutgeren,'ESTRUTURA', null, null, null, null, null, $nrestrutsind);
        $nrpessoas  = array();
        $nrpessoasvinc = array();
        $tipopessoa = '';
        $verificapessoas = '';
        foreach ($resultado as $rows) {
            $verificapessoas = 'A';
            $nrpessoas = $rows['NRPESSOA'];
            $param = array($nrorg,$dtmescompetenc,$nrorgpadrao,$nrpessoas, 'S');
            $result = $this->pessoaService->getPessoaRelacionamentos($param, "");
            foreach($result as $row){
                 if($tipobeneficiario === 'A'){ 
                    if($rows['NMVINCULOM'] === $row['NMPESSOA']){ 
                        $tipopessoa = 'T';
                    }else{
                        $tipopessoa = 'D';
                    }
                    $verificapessoas = '';
                    $nrpessoavinc[] = array('NRVINCULOM' => $rows['NRVINCULOM'], 'NRPESSOA'=> $row['NRPESSOA'],'TIPOPESSOA'=>$tipopessoa,'DTNASCIMENTO' => $row['DTNASCIMENTO'],'NOMEPESSOA' => $row['NMPESSOA']);
                }else if($tipobeneficiario === 'T'){
                    $tipopessoa = 'T';
                    if($rows['NMVINCULOM'] === $row['NMPESSOA']){
                        $verificapessoas = '';
                        $nrpessoavinc[] = array('NRVINCULOM' => $rows['NRVINCULOM'], 'NRPESSOA'=> $row['NRPESSOA'],'TIPOPESSOA'=>$tipopessoa,'DTNASCIMENTO' => $row['DTNASCIMENTO'],'NOMEPESSOA' => $row['NMPESSOA']);
                    }
                }else{
                    if($rows['NMVINCULOM'] !== $row['NMPESSOA']){
                        $tipopessoa = 'D';
                        $verificapessoas = '';
                        $nrpessoavinc[] = array('NRVINCULOM' => $rows['NRVINCULOM'], 'NRPESSOA'=> $row['NRPESSOA'],'TIPOPESSOA'=>$tipopessoa,'DTNASCIMENTO' => $row['DTNASCIMENTO'],'NOMEPESSOA' => $row['NMPESSOA']);
                    }
                }
            }
        }
        if($verificapessoas === 'A'){
            throw new \Exception(Messages::HCM00000_GER_VINCS_SEM_DEPENDENTES_ERROR,Messages::ERROR_MESSAGE_CODE);
        }
        if(empty($nrpessoas)){
            throw new \Exception(Messages::HCM00000_GER_VINCS_N_ENCONT_ESTRUTS_ERROR,Messages::ERROR_MESSAGE_CODE);
        }
        return $nrpessoavinc;
    }
    
    public function filtrarVinculoDependentePorVinculo($nrorg,$dtmescompetenc,$nrorgpadrao,$lista_nrpessoasvinc,$tipobeneficiario){
        $nrpessoasvinc = array();
        $tipopessoa = '';
        $verificapessoas = '';
        foreach($lista_nrpessoasvinc as $rows){
            $verificapessoas = 'A';
            $param = array($nrorg,$dtmescompetenc,$nrorgpadrao,$rows['NRPESSOA'],'S');
            $result = $this->pessoaService->getPessoaRelacionamentos($param, "");
            foreach($result as $row){
                if($tipobeneficiario === 'A'){ 
                    if( $row['NRPESSOA'] === $row['NRPESSOAPRINC'] ){ 
                        $tipopessoa = 'T';
                    }else{
                        $tipopessoa = 'D';
                    } 
                    $verificapessoas = '';
                    $nrpessoavinc[] = array('NRVINCULOM' => $rows['NRVINCULOM'], 'NRPESSOA'=> $row['NRPESSOA'],'TIPOPESSOA'=>$tipopessoa,'DTNASCIMENTO' => $row['DTNASCIMENTO'],'NOMEPESSOA' => $row['NMPESSOA']);
                }else if($tipobeneficiario === 'T'){
                    $tipopessoa = 'T';
                    $verificapessoas = '';
                    if($rows['NMVINCULOM'] === $row['NMPESSOA']){
                        $nrpessoavinc[] = array('NRVINCULOM' => $rows['NRVINCULOM'], 'NRPESSOA'=> $row['NRPESSOA'],'TIPOPESSOA'=>$tipopessoa,'DTNASCIMENTO' => $row['DTNASCIMENTO'],'NOMEPESSOA' => $row['NMPESSOA']);
                    }
                }else{
                    if($rows['NMVINCULOM'] !== $row['NMPESSOA']){
                        $tipopessoa = 'D';
                        $verificapessoas = '';
                        $nrpessoavinc[] = array('NRVINCULOM' => $rows['NRVINCULOM'], 'NRPESSOA'=> $row['NRPESSOA'],'TIPOPESSOA'=>$tipopessoa,'DTNASCIMENTO' => $row['DTNASCIMENTO'],'NOMEPESSOA' => $row['NMPESSOA']);
                    }
                }
            }
        }
        if($verificapessoas === 'A'){
            throw new \Exception(Messages::HCM00000_GER_VINCS_SEM_DEPENDENTES_ERROR,Messages::ERROR_MESSAGE_CODE);
        }
        return $nrpessoavinc;
    }
    
    public function calculavrtabela($dtnascimento,$tabela){
        $vrbeneficio = 0;
        if(empty($dtnascimento)){
                throw new \Exception(Messages::HCM00000_GER_DTNASC_PESS_N_INF_ERROR,Messages::ERROR_MESSAGE_CODE);
        }
        $interval = DateUtil::getDataDeString(substr($dtnascimento,0,10))->diff($this->now);
        $idade = $interval->format( '%Y' );
        foreach($tabela as $row){
            if($idade >= $row->getVrlimitsupfaixa() && $idade <= $row->getVrmaxresultcalc()){
                $percentCalc = $row->getVrpecalculoresu();
                $vraplicaresult = $row->getVraplicaresult();
                if(isset($percentCalc) && !empty($percentCalc)){
                    $vrbeneficio += $vraplicaresult*($percentCalc/100);    
                }else{
                    $vrbeneficio += $vraplicaresult;    
                }
            }
        }
        return $vrbeneficio;
    }
    
    public function subtrairVrEventoFixoVincEvento($nreventom,$nrestruturam,$nrvinculom,$valor,$qtbeneficio){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $getVincEvento = FpaVincevento::findByNreventomAndNrestruturamAndNrvinculomAndNrorg($nreventom,$nrestruturam,$nrvinculom,$nrorg);
        if(empty($getVincEvento)){
            return false;
        }
        foreach ($getVincEvento as $row) {
            $vrfixo = $row->getVreventofixo();
            $vrfixo -= $valor;
            if($vrfixo == 0){
                $row->delete();
            }else{
                $row->setVreventofixo($vrfixo);
                $this->entityManager->persist($row);
            }
            $this->entityManager->flush();
        }
    }

    public function salvarSomarVrEventoFixoVincEvento($nreventom,$nrestruturam,$nrvinculom,$valor,$vrbeneficioOld, $parametrossalvar = null, $editarbeneficiario = null){
        try {
            $nrorg = $this->hcmParameters->getNrorgTrabalho();
            $getVincEvento = FpaVincevento::findByNreventomAndNrestruturamAndNrvinculomAndNrorg($nreventom, $nrestruturam, $nrvinculom, $nrorg);
            if (count($getVincEvento) < 1) {
                $parametrossalvar['VRBENEFICIO'] = $parametrossalvar['VRBENEFICIO'] * $parametrossalvar['QTBENEFICIO'];
                $parametrossalvar['QTREFERENCIA'] = null;
                $parametrossalvar['NRVALORREFERENCIAM'] = null;
                $parametrossalvar['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();;
                $nrvincevento = $this->inserirVincEvento($parametrossalvar);
            } else {
                foreach ($getVincEvento as $row) {
                    $nrvincevento = $row->getNrvincevento();
                    $vrfixo = $row->getVreventofixo();
                    if ($editarbeneficiario === 'S') {
                        $vrfixo -= $vrbeneficioOld;
                        $vrfixo += $valor;
                    } else {
                        $vrfixo += $valor * $parametrossalvar['QTBENEFICIO'];
                    }
                    $row->setVreventofixo($vrfixo);
                    $this->entityManager->persist($row);
                    $this->entityManager->flush();
                }
            }
        } catch (\Exception $e) {
            throw $e;
        }

        return $nrvincevento;
    }

    public function subtrairVrRefFixoVincEvento($nreventom,$nrestruturam,$nrvinculom,$qtbeneficio){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $getVincEvento = FpaVincevento::findByNreventomAndNrestruturamAndNrvinculomAndNrorg($nreventom,$nrestruturam,$nrvinculom,$nrorg);
        if(empty($getVincEvento)){
            return false;
        }
        foreach ($getVincEvento as $row) {
            $nrvincevento = $row->getNrvincevento();
            $vrfixo = $row->getVrreffixo();
            $vrfixo -= $qtbeneficio;
            $row->setVrreffixo($vrfixo);
            $row->setVrmultiplic($vrfixo);
            if($vrfixo == 0){
                $row->delete();
            }else{
                $this->entityManager->persist($row);
            }
            $this->entityManager->flush();
        }
    }

    public function salvarSomarVrRefFixoVincEvento($nreventom,$nrestruturam,$nrvinculom,$parametrossalvar){
        $nrorg = $this->hcmParameters->getNrorgTrabalho();
        $getVincEvento = FpaVincevento::findByNreventomAndNrestruturamAndNrvinculomAndNrorg($nreventom,$nrestruturam,$nrvinculom,$nrorg);
        if(count($getVincEvento) < 1){
            $parametrossalvar['VRBENEFICIO'] = null;
            $parametrossalvar['QTREFERENCIA'] = $parametrossalvar['QTBENEFICIO'];
            $parametrossalvar['DTMESCOMPETENC'] = $this->hcmParameters->getDtCompetencia();;
            $nrvincevento = $this->inserirVincEvento($parametrossalvar);
        }else{
            foreach ($getVincEvento as $row) {
                $nrvalorreferenciaOld = $row->getNrvalorreferenciam();
                if($nrvalorreferenciaOld !== $parametrossalvar['NRVALORREFERENCIAM']){
                    return '!REFERENCIA';
                }
                $nrvincevento = $row->getNrvincevento();
                $vrfixo = $row->getVrreffixo();
                $vrfixo += $parametrossalvar['QTBENEFICIO'];
                $percent = $row->getVrpercent();
                $row->setVrreffixo($vrfixo);
                $row->setVrmultiplic($vrfixo);
                $row->setVrpercent(100);
                $this->entityManager->persist($row);
                $this->entityManager->flush();
            }
        }
        return $nrvincevento;
    }
    
    public function validaCamposVaziosBeneficio(&$row){
        $row['DTFINALVIGENCIA'] = (!isset($row['DTFINALVIGENCIA']) || empty($row['DTFINALVIGENCIA'])) ? NULL : $row['DTFINALVIGENCIA'];
        $row['DTREFERENCIA'] = (!isset($row['DTREFERENCIA']) || empty($row['DTREFERENCIA'])) ? NULL : $row['DTREFERENCIA'];
        $row['NREVENTOMENSA'] = (!isset($row['NREVENTOMENSA']) || empty($row['NREVENTOMENSA'])) ? NULL : $row['NREVENTOMENSA'];
        $row['NREVENTOMENSADEP'] = (!isset($row['NREVENTOMENSADEP']) || empty($row['NREVENTOMENSADEP'])) ? NULL : $row['NREVENTOMENSADEP'];
        $row['NREVETITULAREMP'] = (!isset($row['NREVETITULAREMP']) || empty($row['NREVETITULAREMP'])) ? NULL : $row['NREVETITULAREMP'];
        $row['NREVEDEPEMPRESA'] = (!isset($row['NREVEDEPEMPRESA']) || empty($row['NREVEDEPEMPRESA'])) ? NULL : $row['NREVEDEPEMPRESA'];
        $row['NREVETITULARVINC'] = (!isset($row['NREVETITULARVINC']) || empty($row['NREVETITULARVINC'])) ? NULL : $row['NREVETITULARVINC'];
        $row['NREVEDEPEVINC'] = (!isset($row['NREVEDEPEVINC']) || empty($row['NREVEDEPEVINC'])) ? NULL : $row['NREVEDEPEVINC'];
        $row['NRTABFAIXASALTIT'] = (!isset($row['NRTABFAIXASALTIT']) || empty($row['NRTABFAIXASALTIT'])) ? NULL : $row['NRTABFAIXASALTIT'];
        $row['NRTABFAIXASALDEP'] = (!isset($row['NRTABFAIXASALDEP']) || empty($row['NRTABFAIXASALDEP'])) ? NULL : $row['NRTABFAIXASALDEP'];
        $row['NRMESANIVERSARIO'] = (!isset($row['NRMESANIVERSARIO']) || empty($row['NRMESANIVERSARIO'])) ? NULL : $row['NRMESANIVERSARIO'];
        $row['NRDIAVENCBOLETO'] = (!isset($row['NRDIAVENCBOLETO']) || empty($row['NRDIAVENCBOLETO'])) ? NULL : $row['NRDIAVENCBOLETO'];
        $row['VRLINEARBENEFICIO'] = (!isset($row['VRLINEARBENEFICIO']) || empty($row['VRLINEARBENEFICIO'])) ? NULL : $row['VRLINEARBENEFICIO'];
        $row['VRLINEARBENEFICIODEP'] = (!isset($row['VRLINEARBENEFICIODEP']) || empty($row['VRLINEARBENEFICIODEP'])) ? NULL : $row['VRLINEARBENEFICIODEP'];
        $row['NRIDADEMAXSUBFILHOS'] = (!isset($row['NRIDADEMAXSUBFILHOS']) || empty($row['NRIDADEMAXSUBFILHOS'])) ? NULL : $row['NRIDADEMAXSUBFILHOS'];
        $row['NRIDADEMAXSUBDEP'] = (!isset($row['NRIDADEMAXSUBDEP']) || empty($row['NRIDADEMAXSUBDEP'])) ? NULL : $row['NRIDADEMAXSUBDEP'];
        $row['NRTIPOPARTICIPATITULAR'] = (!isset($row['NRTIPOPARTICIPATITULAR']) || empty($row['NRTIPOPARTICIPATITULAR'])) ? NULL : $row['NRTIPOPARTICIPATITULAR'];
        $row['VRPARTICIPATITULAR'] = (!isset($row['VRPARTICIPATITULAR']) || empty($row['VRPARTICIPATITULAR'])) ? NULL : $row['VRPARTICIPATITULAR'];
        $row['NRTIPOPARTICIPADEP'] = (!isset($row['NRTIPOPARTICIPADEP']) || empty($row['NRTIPOPARTICIPADEP'])) ? NULL : $row['NRTIPOPARTICIPADEP'];
        $row['VRPARTICIPADEPE'] = (!isset($row['VRPARTICIPADEPE']) || empty($row['VRPARTICIPADEPE'])) ? NULL : $row['VRPARTICIPADEPE'];
        $row['VRPISOTITULAR'] = (!isset($row['VRPISOTITULAR']) || empty($row['VRPISOTITULAR'])) ? NULL : $row['VRPISOTITULAR'];
        $row['VRTETOTITULAR'] = (!isset($row['VRTETOTITULAR']) || empty($row['VRTETOTITULAR'])) ? NULL : $row['VRTETOTITULAR'];
        $row['VRPISODEP'] = (!isset($row['VRPISODEP']) || empty($row['VRPISODEP'])) ? NULL : $row['VRPISODEP'];
        $row['VRTETODEP'] = (!isset($row['VRTETODEP']) || empty($row['VRTETODEP'])) ? NULL : $row['VRTETODEP'];
        $row['VRPISOTITULARSIND'] = (!isset($row['VRPISOTITULARSIND']) || empty($row['VRPISOTITULARSIND'])) ? NULL : $row['VRPISOTITULARSIND'];
        $row['VRTETOTITULARSIND'] = (!isset($row['VRTETOTITULARSIND']) || empty($row['VRTETOTITULARSIND'])) ? NULL : $row['VRTETOTITULARSIND'];
        $row['VRPISODEPSIND'] = (!isset($row['VRPISODEPSIND']) || empty($row['VRPISODEPSIND'])) ? NULL : $row['VRPISODEPSIND'];
        $row['VRTETODEPSIND'] = (!isset($row['VRTETODEPSIND']) || empty($row['VRTETODEPSIND'])) ? NULL : $row['VRTETODEPSIND'];
        $row['PERCOPARTICIPATITU'] = (!isset($row['PERCOPARTICIPATITU']) || empty($row['PERCOPARTICIPATITU'])) ? NULL : $row['PERCOPARTICIPATITU'];
        $row['PERCOPARTICIPADEPE'] = (!isset($row['PERCOPARTICIPADEPE']) || empty($row['PERCOPARTICIPADEPE'])) ? NULL : $row['PERCOPARTICIPADEPE'];
        $row['VRTITULARSIND'] = (!isset($row['VRTITULARSIND']) || empty($row['VRTITULARSIND'])) ? NULL : $row['VRTITULARSIND'];
        $row['VRDEPSIND'] = (!isset($row['VRDEPSIND']) || empty($row['VRDEPSIND'])) ? NULL : $row['VRDEPSIND'];
        $row['NREVENTOTAXA'] = (!isset($row['NREVENTOTAXA']) || empty($row['NREVENTOTAXA'])) ? NULL : $row['NREVENTOTAXA'];
        $row['VREVENTOTAXA'] = (!isset($row['VREVENTOTAXA']) || empty($row['VREVENTOTAXA'])) ? NULL : $row['VREVENTOTAXA'];
        $row['NRDIARESCISAO'] = (!isset($row['NRDIARESCISAO']) || empty($row['NRDIARESCISAO'])) ? NULL : $row['NRDIARESCISAO'];
        $row['IDDUPLICMENS'] = (!isset($row['IDDUPLICMENS']) || empty($row['IDDUPLICMENS'])) ? NULL : $row['IDDUPLICMENS'];
        $row['IDPROPDTCANC'] = (!isset($row['IDPROPDTCANC']) || empty($row['IDPROPDTCANC'])) ? NULL : $row['IDPROPDTCANC'];
        $row['IDPROPDTINSCR'] = (!isset($row['IDPROPDTINSCR']) || empty($row['IDPROPDTINSCR'])) ? NULL : $row['IDPROPDTINSCR'];
        $row['VRPISOTITULARMENS'] = (!isset($row['VRPISOTITULARMENS']) || empty($row['VRPISOTITULARMENS'])) ? NULL : $row['VRPISOTITULARMENS'];
        $row['VRTETOTITULARMENS'] = (!isset($row['VRTETOTITULARMENS']) || empty($row['VRTETOTITULARMENS'])) ? NULL : $row['VRTETOTITULARMENS'];
        $row['NRTABELAFOLHATERC'] = (!isset($row['NRTABELAFOLHATERC']) || empty($row['NRTABELAFOLHATERC'])) ? NULL : $row['NRTABELAFOLHATERC'];
        $row['NRTABELAFOLHAFOUR'] = (!isset($row['NRTABELAFOLHAFOUR']) || empty($row['NRTABELAFOLHAFOUR'])) ? NULL : $row['NRTABELAFOLHAFOUR'];
        $row['QTDPARCELAS'] = (!isset($row['QTDPARCELAS']) || empty($row['QTDPARCELAS'])) ? NULL : $row['QTDPARCELAS'];
        $row['NRVALORREFERENCIAM'] = (!isset($row['NRVALORREFERENCIAM']) || empty($row['NRVALORREFERENCIAM'])) ? NULL : $row['NRVALORREFERENCIAM'];
    }
    
    public function validaCamposVaziosBeneficiario(&$row){
        $row['NREVENTOM'] = NULL;
        $row['IDTPBENEFICIARIO'] = (!isset($row['IDTPBENEFICIARIO']) || empty($row['IDTPBENEFICIARIO'])) ? NULL : $row['IDTPBENEFICIARIO'];
        $row['IDVALORDIFEBENEF'] = NULL;
        $row['NRTPBENEFICIO'] = NULL;
        $row['NRINTEGTRANSPM'] = (!isset($row['NRINTEGTRANSPM']) || empty($row['NRINTEGTRANSPM'])) ? NULL : $row['NRINTEGTRANSPM'];
        $row['NRVALORREFERENCIAM'] = NULL;
        $row['NRVINCEVENTO'] = NULL;
        $row['VRBENEFICIO'] = NULL;
        $row['SUBSIDIO100'] = (!isset($row['SUBSIDIO100']) || empty($row['SUBSIDIO100'])) ? 'N' : $row['SUBSIDIO100'];
        $row['VRPERCSUBSIDIO'] = (!isset($row['VRPERCSUBSIDIO']) || empty($row['VRPERCSUBSIDIO'])) ? NULL : $row['VRPERCSUBSIDIO'];
    }
    
    public function validaCamposVaziosBeneficiarioPorEvento(&$row){
        $row['VRPERCSUBSIDIO'] = (!isset($row['VRPERCSUBSIDIO']) || empty($row['VRPERCSUBSIDIO'])) ? NULL : $row['VRPERCSUBSIDIO'];
    }
}  