<?php
/**
 * C:\workfolder\paulopereira\products\v_200\packages\Estoque\Logic\FechamentoDia\FechamentoDia.php
 *
 * Created by JetBrains PhpStorm.
 * User: erikasouza
 * Date: 16/10/12
 * Time: 14:16
 * To change this template use File | Settings | File Templates.
 */

namespace Estoque\Logic\FechamentoDia;

use Estoque\Entities\Diainvefili;
use Estoque\Entities\Inverota;
use Estoque\Entities\Inverotalote;
use Estoque\Entities\Itlanctoest;
use Estoque\Logic\FechamentoDia\Util;
use Estoque\Logic\FechamentoDia\Inventario;
use Estoque\Util\Parametros;
use \Fiscal\Entities\Notafiscal;
use \Geral\Entities\Inventestcego;
use \Geral\Entities\Iteminventcego;
use \Geral\Entities\Localestoque;
use \Geral\Entities\Paramfilial;
use \Geral\Entities\Filial;
use \Geral\Util\Ambiente;
use \Geral\Util\DateUtil;
use \Geral\Entities\Paramestoque;
use \Geral\Entities\Ferifili;
use \Planejamento\Entities\Cardadia;
use \Indeterminado\Entities\Parperdivestceg;
use \Estoque\Logic\Exception\ExceptionFechamentoDia;
use Geral\Repositories\Retirada;
use \Estoque\Repositories\Posestdia;
use Setforms2\Shared\ORM\DateTime;

class FechamentoDia {


    const FORMATEDATE_EN = 'y-MM-dd';
    const FORMATEDATE = 'Y-m-d';
    const IDALTDTPROCFDIA = 'S';
    /** IDALTDTPROCFDIA = S */
    const DATA_ALTERADA = 'S';
    /** IDALTDTPROCFDIA = N */
    const DATA_NAO_ALTERADA = 'N';

    private $operador;
    private $nrorg;

    private $dataAtual;
    private $diaDtProc;

    private $realizaInventDia;
    private $percentual;
    private $geraInvRot;
    private $utilInvRotRequisicao;

    private $cdAlmox;
    private $utilAlmox;
    private $cdLocal;
    private $utilLocal;
    private $utilLote;

    private $consDiaNaoUtil;
    private $bloqFechDia;
    private $bloqFechamRetiradas;
    private $bloqFechamNF;
    private $emissaoRE;
    private $bloqFechSaidaNF;
    private $bloqEstNeg;


    public function __construct($dtProc){
        /* Ambiente */
        $this->operador = Ambiente::getCdOperador();
        $this->nrorg    = Ambiente::getNrorg();

        /* Datas */
        $dataAtual       = DateUtil::getDataAtual();
        $this->dataAtual = $dataAtual->format(DateUtil::FORMATO_BRASILEIRO);

        $this->diaDtProc = DateUtil::getDia(DateUtil::getDataDeString($dtProc));

        /* Paramêtros inventario */
        $this->realizaInventDia = Parametros::getInventarioRotativoDiariamente();
        $this->percentual = Parametros::getPercentualInventarioRotativo();
        $this->geraInvRot = Parametros::getGeraInventarioRotativo();
        $this->utilInvRotRequisicao = Parametros::getInventarioRotativoRequisicao();

        /* Paramentros Filial */
        $this->cdAlmox = Parametros::getAlmoxarifado();
        $this->utilAlmox = Parametros::getUtilizaAlmoxarifado();
        $this->cdLocal = Parametros::getLocalEstoque();
        $this->utilLocal = Parametros::getUtilizaLocalizacao();
        $this->utilLote = Parametros::getUtilizaLote();

        /*Paramentros Fechamento Dia */
        $this->consDiaNaoUtil = Parametros::getConsideraDiaNaoUtilFech();
        $this->bloqFechDia = Parametros::getBloqueiaFechamentoDia();
        $this->bloqFechamRetiradas = Parametros::getBloqueioSemRetirada();
        $this->bloqFechamNF = Parametros::getBloqueioSemNF();
        $this->emissaoRE = Parametros::getObrigaEmissaoRE();
        $this->bloqFechSaidaNF = Parametros::getBloqueioFechamentoSaidaNF();
        $this->bloqEstNeg = \Geral\Repositories\Paramfilial::consultaEstoqueNegativo(Ambiente::getCdFilial());
    }


    /**
     * Parte I do Fechamento do dia.
     *
     * @param $filiProc
     * @param $dtProc
     * @param $proxDtProc
     *
     * @return array|bool|string
     */
    public function validaInventario($filiProc, $dtProc, $proxDtProc){
        $inventarioPendente = $this->verificaInventarioPendente($this->nrorg, $filiProc, $dtProc);
        if ($inventarioPendente) {
            /** @var $inventario Inventestcego */
            foreach ($inventarioPendente as $inventario) {
                $divergencia = $this->verificaDiferencaEstoqueDiaInvent (
                    $this->nrorg, $filiProc, $inventario->getDtcontagem()->format(self::FORMATEDATE)
                );

                if ($divergencia) {
                    ExceptionFechamentoDia::errodivergencia();
                }
            }
        }

        $estoqueCego = $this->validaEstoqueCego($this->nrorg, $filiProc, $dtProc, $proxDtProc);
        if ($estoqueCego === false) {
            ExceptionFechamentoDia::erroEstoqueCego();
        } else if (is_string($estoqueCego) === true){
            return array(
                'MENSAGEM' => 'fechDia_confEstoqueCego',
                'PARAM' => array($estoqueCego)
            );
        }

        return true;
    }

    /**
     * Parte II do Fechamento do dia. (ConfirmaÃ§Ã£o do operador para continuaÃ§Ã£o.)
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     * @param $dtProxDtProces
     *
     * @return array|bool|string]
     */
    public function preValidaFechDia($nrorg, $filial, $dtProc, $dtProxDtProces) {
        $this->verificaEfetivos($nrorg, $filial, $dtProc);
        $this->verificaRetirada($nrorg, $filial, $dtProc);
        $this->verificaEntradaSemNF($nrorg, $filial, $dtProc);

        $fechamentoMes = new FechamentoMes();
        $fechamentoMes->verificaPosicaoEstoque($nrorg, $filial, $dtProc, $dtProxDtProces);

        // ValidaÃ§Ã£o do Almoxarifado, da LocalizaÃ§Ã£o e do dia do Fechamento do MÃªs.
        $this->validaParametrizacao($nrorg, $filial, $dtProc);
        return $this->validaInventarioRotativo($nrorg, $filial, $dtProc);
    }

    /**
     * Realiza as validações para fechar o dia com nova data de processamento
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     * @param $dtProxDtProces
     *
     * @return array|bool|string]
     */
    public function validaFechDia($nrorg, $filial, $dtProc, $dtProxDtProces) {
    	$this->verificaEfetivos($nrorg, $filial, $dtProc);
    	$this->verificaRetiradaEstoque($filial, $dtProc);
    	$this->verificaRetiradaSemNF($nrorg, $filial, $dtProc);
    	$this->verificaEntradaSemNF($nrorg, $filial, $dtProc);
    	$this->verificaNFSaida($filial, $dtProc);
    	$this->verificaProdutoIrregular($filial, $dtProc, $dtProxDtProces);
    	//Validação do Almoxarifado, da Localização e do dia do Fechamento do Mês.
    	$this->validaParametrizacao($nrorg, $filial, $dtProc);
		$this->validaInventarioRotativo($nrorg, $filial, $dtProc);
    }


    public function transacaoFechamentoDia($filial, $dtProc, $dtProxDtProces) {
        $connection = \Setforms2\Shared\Registry::getEntityManager();
        $connection->beginTransaction();

        try {
            $fechamentoMes = new FechamentoMes();
            $fechamentoMes->transacaoFechamentoMes(
                $this->nrorg, $filial, $this->operador, $dtProc, $dtProxDtProces, $this->utilAlmox, $this->utilLote,
                $this->cdAlmox, $this->cdLocal, $this->realizaInventDia
            );

            $dtInvent = Diainvefili::consultaDiaInventariado($this->nrorg, $filial);
            $inventario = new Inventario();

            /** @var $localestoqueRepository \Geral\Repositories\Localestoque */
            $localestoqueRepository = Localestoque::getRepository();
            $dados = $localestoqueRepository->consultaAlmoxarifadoLocalizacaoPorFilial($this->nrorg, $filial);
            if ($dados) {
                /** @var $dado \Geral\Entities\Localestoque */
                foreach ($dados as $dado) {
                    $almox = $dado->getAlmoxarife()->getCdalmoxarife();
                    $local = $dado->getCdlocalestoq();

                    if ($this->realizaInventDia === 'S' OR $dtInvent === $this->diaDtProc) {
                        $inventario->inventarioNormal(
                            $this->nrorg, $filial, $dtProc, $this->utilLote, $almox, $local,
                            $this->dataAtual, $this->operador, $this->realizaInventDia
                        );

                        $inventario->inventarioPosicaoEstoque(
                            $this->nrorg, $filial, $dtProc, $this->utilLote, $this->dataAtual, $this->operador,
                            $this->cdAlmox, $this->cdLocal
                        );

                        $inventario->inventarioRequisicao(
                            $this->nrorg, $filial, null, $this->dataAtual, $dtProc, $this->cdAlmox, $this->cdLocal,
                            $this->utilLote, $this->realizaInventDia
                        );
                    } else {
                        $inventario->inventarioNormalProxy(
                            $this->nrorg, $filial, $dtProc, $this->utilLote, $almox, $local,
                            $this->dataAtual, $this->operador, $this->realizaInventDia
                        );
                    }
                }
            }

            $exportacaoPosFechamento = new ExportPosFechamentoDia($filial, $this->nrorg, $dtProc);

            try{
               $exportacaoPosFechamento->exportacaoPosFechamentoDia($this->nrorg, $filial, $dtProc, $dtProxDtProces);
            } catch (\Exception $e) {
                throw $e;
            }


            /** @var $objParamFilial Paramfilial */
            $objParamFilial = Paramfilial::getObject(
                array(
                    'nrorg' => $this->nrorg,
                    'cdfilial' => $filial
                )
            );

            $proxDtProc = \Geral\Util\DateUtil::getDataDeString($dtProxDtProces);
            $objParamFilial->atualizaDataProcessamento($proxDtProc);
            $connection->flush();

            $this->validaFechamentoDia($this->nrorg, $filial, $this->utilLote, $dtProc);

            $connection->flush();
            $connection->commit();

            // ExceptionFechamentoDia::sucessoFechamentoOK();
        } catch (\Exception $e) {
            $connection->rollback();
            \Geral\Util\TratamentoDeMensagens::exibirMensagens($e);
        }
    }


    /* INICIA O FECHAMENTO DO DIA */


    /**
     * Verifica se há inventarios pendentes com data igual a maior data anterior a data de processamento.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     *
     * @return mixed
     */
    public function verificaInventarioPendente($nrorg, $filial, $dtProc){
        /** @var $obj \Geral\Entities\Inventestcego */
        $obj = Inventestcego::getObjectByWhere(
            'NRORG = ? AND CDFILIAL = ? AND DTCONTAGEM < ?',
            array($nrorg, $filial, $dtProc,),
            'DTCONTAGEM DESC'
        );

        if ($obj) {
            $dataContagem = $obj->getDtcontagem()->format(self::FORMATEDATE);

            /** @var $entityRepository \Geral\Repositories\Inventestcego */
            $entityRepository = Inventestcego::getRepository();
            $inventPendente = $entityRepository->verificaInventarioPendente($nrorg, $filial, $dataContagem);
            return $inventPendente;
        }

        return false;
    }


    /**
     * Verifica em quais produtos ocorre diferença de quantidade.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtContagem
     *
     * @return array
     */
    public function verificaDiferencaEstoqueDiaInvent($nrorg, $filial, $dtContagem){
        $data = DateUtil::getPrimeiroDiaDoMes(DateUtil::getDataDeString($dtContagem));
        $dtAntContagem = $data->format(DateUtil::FORMATO_BRASILEIRO);

        /** @var $entityRepository \Geral\Repositories\Inventestcego */
        $entityRepository = Inventestcego::getRepository();
        $diferenca        = $entityRepository->verificaDiferencaEstoqueDiaInventario(
            $nrorg, $filial, $dtContagem, $dtAntContagem
        );

        return $diferenca;
    }


    /**
     * Normaliza a quantidade de produtos no estoque do dia com o inventário e valida a continuação do
     * fechamento do dia.
     *
     * @param $nrorg
     * @param $filiProc
     * @param $dtProc
     * @param $proxDtProc
     *
     * @return bool
     *
     * @throws \Exception Caso ocorra erro na atualização da quantidade de produto no estoque.
     */
    public function validaEstoqueCego($nrorg, $filiProc, $dtProc, $proxDtProc){
        /** @var $inventestcegoRepository \Geral\Repositories\Inventestcego */
        $inventestcegoRepository = Inventestcego::getRepository();
        $inventAbertos = $inventestcegoRepository->verificaInventarioAberto($nrorg, $filiProc, $dtProc, $proxDtProc);

        if ($inventAbertos) {
            foreach ($inventAbertos as $inventario) {
                $dtContagem = $inventario['DTCONTAGEM'];
                $dtContagem = explode(" ", $dtContagem);
                $dtContagem = $dtContagem[0];

                $bloqueio = $inventario['IDBLOQFECINVPE'];
                $situacao = $inventario['IDSITESTCEGO'];

                // Verifica quais produtos têm diferença na quantidade no estoque do dia em relação ao inventário.
                $diferencaProduto = $this->verificaDiferencaEstoqueDiaInvent($nrorg, $filiProc, $dtContagem);

                if ($diferencaProduto) {
                    $connection = \Setforms2\Shared\Registry::getEntityManager();
                    $connection->beginTransaction();
                    foreach ($diferencaProduto as $produto) {
                        $cdArvProd = $produto['CDARVPROD'];
                        $qtProdInvent = $produto['QTINVENTESTCEGO'];
                        $qtProdDia = $produto['QTESTOQDIA'];
                        $cdAlmoxarife = $produto['CDALMOXARIFE'];
                        $cdLocalEstoq = $produto['CDLOCALESTOQ'];
                        $nrloteestq = $produto['NRLOTEESTQ'];
                        $nrSubLote = $produto['NRSUBLOTE'];

                        $toleranciaProd = Parperdivestceg::getObjectsByWhere(
                            'CDFILIAL = ? AND NRORG = ? AND ? BETWEEN CDPRODUTO AND CDPRODUTO ||\'zzzzzzzzzz\'',
                            array($filiProc, $nrorg, $cdArvProd)
                        );

                        if ($toleranciaProd) {
                            /** @var $dados Parperdivestceg */
                            foreach ($toleranciaProd as $dados) {
                                // abs = Valor Absoluto
                                $tolerancia = abs(($qtProdInvent * 100 / $qtProdDia) - 100);
                                $percentual_tolerancia = $dados->getVrperdivestceg();
                                if ($tolerancia >= $percentual_tolerancia) {
                                    try {
                                        // $data = new \DateTime($dtContagem);
                                        $data = DateUtil::getDataDeString($dtContagem);
                                        /** @var $objItem Iteminventcego */
                                        $objItem = Iteminventcego::getObject(
                                            array(
                                                'nrorg' => $nrorg,
                                                'cdfilial' => $filiProc,
                                                'dtcontagem' => $data,
                                                'cdalmoxarife' => $cdAlmoxarife,
                                                'cdlocalestoq' => $cdLocalEstoq,
                                                'nrloteestq' => $nrloteestq,
                                                'nrsublote' => $nrSubLote,
                                                'cdproduto' => $cdArvProd,
                                            )
                                        );
                                        $objItem->setQtposestinvcego($qtProdDia);
                                        $objItem->persist();

                                        \Setforms2\Shared\Registry::getEntityManager()->flush();
                                        $connection->commit();
                                    } catch (ExceptionFechamentoDia $e) {
                                        $connection->rollback();
                                        ExceptionFechamentoDia::erroAtualizaEstoqueCego();
                                    }
                                }

                                // De acordo com a situação do inventario pede ao usuário a confirmação
                                // para continuar com o Fechamento do dia.
                                if ($situacao === 'PR' OR $situacao === 'DG' OR $situacao === 'ED') {
                                    if ($bloqueio === 'N') {
                                        return $situacao;
                                    } else {
                                        return false;
                                    }
                                }
                            }
                        }
                    }
                }
            }
        }

        return true;
    }


    /* FIM FECHAMENTO DO DIA */


    /* INICIO CONFIRMAÇÃO DO FECHAMENTO DO DIA */


    /**
     * Verifica se há registro de clientes que fizeram a refeição no dia.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProcesso
     *
     * @throws \Exception
     * @return bool|string
     */
    public function verificaEfetivos($nrorg, $filial, $dtProcesso){
        try {
            if (Parametros::getVerificaEfetivos() === 'S') {
                /** @var $cardadiaRepository \Planejamento\Repositories\Cardadia */
                $cardadiaRepository = Cardadia::getRepository();
                $efetivos = $cardadiaRepository->verificaExisteEfetivos($nrorg, $filial, $dtProcesso);
                if ($efetivos) {
                    ExceptionFechamentoDia::erroSemEfetivos();
                }
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Verifica se há alguma retirada sem nota fiscal lançada.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProcesso
     *
     * @throws \Exception
     * @return bool|string
     */
    public function verificaRetirada($nrorg, $filial, $dtProcesso){
        try {
            if ($this->bloqFechamRetiradas === 'S') {
                /** @var $itlanctoestRepository  \Estoque\Repositories\Itlanctoest */
                $itlanctoestRepository = Itlanctoest::getRepository();
                $retSemNotas = $itlanctoestRepository->verificaRetiradaSemNotas($nrorg, $filial, $dtProcesso);

                if ($retSemNotas !== false) {
                    ExceptionFechamentoDia::erroRetiradaSemNF();
                }
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica se há alguma retirada sem nota fiscal lançada, de itens que não evoluíram o estoque
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProcesso
     *
     * @throws \Exception
     * @internal param $nrorg
     * @return bool|string
     */
    public function verificaRetiradaSemNF($nrorg, $filial, $dtProcesso){
        try {
            if ($this->bloqFechamRetiradas === 'S') {
                /** @var $itlanctoestRepository  \Estoque\Repositories\Itlanctoest */
                $itlanctoestRepository = Itlanctoest::getRepository();
                $retSemNotas = $itlanctoestRepository->verificaRetiradaSemNotas($nrorg, $filial, $dtProcesso);

                if ($retSemNotas !== false) {
                    ExceptionFechamentoDia::erroRetiradaSemNF();
                }
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica retirada de estoque
     *
     * @param $filial
     * @param $dtProcesso
     * @throws \Exception
     * @internal param $nrorg
     * @return bool
     */

    public function verificaRetiradaEstoque($filial, $dtProcesso){
        try {
            if ($this->bloqFechamNF === 'S') {
                $data = DateUtil::getDataDeString($dtProcesso);
                /** @var $retirada \Geral\Entities\Retirada */
                $retirada = \Geral\Entities\Retirada::findOneByCdfilialAndDtretiradaAndNrorg($filial, $data);

//                $itlanctoestRepository = Itlanctoest::getRepository();
//                $retirada = $itlanctoestRepository->verificaRetirada($filial, $dtProcesso);

                if ($retirada !== false) {
                    ExceptionFechamentoDia::erroSemLancamento();
                }
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica se há alguma entrada sem nota fiscal.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProcesso
     *
     * @throws \Exception
     * @return bool|string
     */
    public function verificaEntradaSemNF($nrorg, $filial, $dtProcesso){
        try {
            if ($this->bloqFechamNF === 'S') {
                /** @var $notaFiscaltRepository  \Fiscal\Repositories\NotaFiscal */
                $notaFiscaltRepository = Notafiscal::getRepository();
                $entSemNotas = $notaFiscaltRepository->verificaEntradaSemNotas($nrorg, $filial, $dtProcesso);
                if ($entSemNotas) {
                    ExceptionFechamentoDia::erroEntradaSemNF();
                }
            }
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica NF de saída não emitidas
     *
     * @param $filial
     * @param $dtProc
     * @throws \Exception
     * @return bool
     */
    public function verificaNFSaida($filial, $dtProc){
        try {
            if ($this->bloqFechSaidaNF === 'S') {
                /** @var $notaFiscaltRepository  \Fiscal\Repositories\NotaFiscal */
                $notaFiscaltRepository = Notafiscal::getRepository();
                $saidaNF = $notaFiscaltRepository->verificaNFSaida($filial, $dtProc);
                if ($saidaNF) {
                    ExceptionFechamentoDia::erroSaidaNaoEmitida();
                }
            }
            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Verifica produtos irregulares
     *
     * @param unknown $filial
     * @param $dtinicial
     * @param $dtfinal
     * @throws \Exception
     *
     * @internal param \Estoque\Logic\FechamentoDia\unknown $nrorg
     * @internal param \Estoque\Logic\FechamentoDia\unknown $dtProcesso
     * @return bool
     */
    public function verificaProdutoIrregular($filial, $dtinicial, $dtfinal){
        try {
            if ($this->bloqEstNeg === 'S') {
                /** @var $posestdiaRepository  \Estoque\Repositories\Posestdia */
                $posestdiaRepository = Posestdia::getRepository();
                $produtoIrregular = $posestdiaRepository->verificaProdutoIrregularII($filial, $dtinicial, $dtfinal);
                if ($produtoIrregular) {
                    ExceptionFechamentoDia::erroProdutoIrregular();
                }
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Validações que obrigam a informação do almoxarifado padrão, localização padrão
     * e dia de fechamento do mês para a filial mediante à alguns parametros.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     *
     * @throws \Exception
     * @return bool|string
     */
    public function validaParametrizacao($nrorg, $filial, $dtProc){
        try {
            if ($this->bloqFechDia === 'S') {
                $fechaMes = new FechamentoMes();
                $dataFech = $fechaMes->consultaDataFechMes($nrorg, $filial, $dtProc);
                if ($dataFech === false) {
                    ExceptionFechamentoDia::erroFechMesSemDt();
                }
            }

            if (empty($this->cdAlmox) === true AND $this->utilAlmox === 'S' AND $this->utilInvRotRequisicao == 'S') {
                ExceptionFechamentoDia::erroSemAlmoxarifado();
            }

            if (empty($this->cdLocal) === true AND $this->utilLocal === 'S' AND $this->utilInvRotRequisicao == 'S') {
                ExceptionFechamentoDia::erroSemLocalizacao();
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }


    /**
     * Verifica se há inventários (com ou sem utilização do lote) pendentes.
     *
     * @param $nrorg
     * @param $filial
     * @param $dtProc
     *
     * @throws \Exception
     * @return array|bool|string
     */
    public function validaInventarioRotativo($nrorg, $filial, $dtProc){
        try {
            if ($this->percentual > 0 OR $this->geraInvRot === 'S' OR $this->utilInvRotRequisicao === 'S') {
                if ($this->utilLote === 'S') {
                    /* @var $inverotaloteRepository \Estoque\Repositories\Inverotalote */
                    $inverotaloteRepository = Inverotalote::getRepository();
                    $inventPendente = $inverotaloteRepository->consultaInventPendenteRotaLote($nrorg, $filial, $dtProc);
                } else {
                    /* @var $inverotaRepository \Estoque\Repositories\Inverota */
                    $inverotaRepository = Inverota::getRepository();
                    $inventPendente = $inverotaRepository->consultaInventPendenteRota($nrorg, $filial, $dtProc);
                }

                if ($inventPendente) {
                    if ($this->utilAlmox === 'S' && $this->utilLocal === 'S') {
                        ExceptionFechamentoDia::erroInventPendAlmoxLocal(array($this->cdAlmox, $this->cdLocal));
                    } else if ($this->utilAlmox === 'S' && $this->utilLocal === 'N') {
                        ExceptionFechamentoDia::erroInventPendentAlmox(array($this->utilAlmox));
                    } else {
                        ExceptionFechamentoDia::erroInventPendFilial();
                    }
                }
            }

            return true;
        } catch (\Exception $e) {
            throw $e;
        }
    }

    /**
     * Valida o fechamento do dia
     *
     * @param $nrorg
     * @param $filial
     * @param $utilLote
     * @param $dtProc
     * @return bool
     * @throws \Exception
     */
    public function validaFechamentoDia($nrorg, $filial, $utilLote, $dtProc){
        try {
            $dados = false;

            if ($utilLote === 'S') {
                $params = array($nrorg, $filial, $dtProc);

                $dados = Inverotalote::getObjectByWhere(
                    'NRORG = ? AND CDFILIAL = ? AND DTINVELOTE = ?',
                    $params
                );
            } else {
                $params = array($nrorg, $filial, $dtProc);

                $dados = Inverota::getObjectByWhere(
                    'NRORG = ? AND CDFILIAL = ? AND DTINVEROTA = ?',
                    $params
                );
            }

            if ($dados === false) {
                /** @var $objParamFilial Paramfilial */
                $objParamFilial = Paramfilial::getObject(
                    array(
                        'nrorg' => $this->nrorg,
                        'cdfilial' => $filial
                    )
                );

                //último passo
                $objParamFilial->atualizaFechamentoDia('S');
                return true;
            } else {
                ExceptionFechamentoDia::erroInventariado();
                return false;
            }
        } catch (\Exception $e) {
            throw $e;
        }
    }


    public function validaRE($filial, $nrorg, $dtProc){
        if ($this->emissaoRE === 'S'){
            /** @var $notaFiscalRepository \Fiscal\Repositories\Notafiscal */
            $notaFiscalRepository = NotaFiscal::getRepository();
            $dadosNFEntrada = $notaFiscalRepository->consultaNotasFiscaisEntrada($filial, $nrorg, $dtProc);
            if ($dadosNFEntrada === false) {
                ExceptionFechamentoDia::erroNFSemEntrada();
            }

            $dadosNFSaida = $notaFiscalRepository->consultaNotasFiscaisSaida($filial, $nrorg, $dtProc);
            if ($dadosNFSaida === false) {
                ExceptionFechamentoDia::erroNFSemSaida();
            }
        }

        return true;
    }
}